try:
    from Meshing import *
    from Element_Stiffness import *
    from Assemble_Stiffness import *
except:
    from KIB_LAP.Scheibe.Meshing import *
    from KIB_LAP.Scheibe.Element_Stiffness import *
    from KIB_LAP.Scheibe.Assemble_Stiffness import *

from matplotlib.collections import PatchCollection
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as patches
from matplotlib.widgets import Slider


import matplotlib.cm as cm
from tabulate import *
from scipy.interpolate import griddata


class ShellCalculation:
    def __init__(self, MeshingParams="Meshing_R"):
        self.Meshing_Mode = MeshingParams
        self.Get_Meshing_Parameters()
        self.Get_Material_Parameters()
        self.Get_Element_Matrices()
        self.Assemble_Stiffness_Matrices()
        self.SolveAssembled_Matrix()
        self.StoreElementDisplacements()
        self.CalculateInnerElementForces_Gauss()

    def BuildMultiPatchMeshFromCSV(
        self, patches_csv="Meshing_Parameters/Meshing_Patches.csv", tol=None
    ):
        df = pd.read_csv(patches_csv)

        # minimale Plausibilitätschecks
        required = {"patch_id", "d1", "d2", "num_x", "num_y", "elem_type", "dx", "dy"}
        miss = required - set(df.columns)
        if miss:
            raise ValueError(f"Missing columns in {patches_csv}: {miss}")

        meshes = []
        for _, row in df.iterrows():
            m = MeshingClass(
                d1=float(row["d1"]),
                d2=float(row["d2"]),
                num_x=int(row["num_x"]),
                num_y=int(row["num_y"]),
                elem_type=str(row["elem_type"]),
            )
            # aktuell: nur rect, analog kannst du später quad-from-csv je patch machen
            if str(row["elem_type"]).lower() == "rect":
                m.generating_rectangular_mesh()
            else:
                raise ValueError("Only 'rect' supported in multipatch example for now.")

            # absolute Verschiebung
            m.translate(dx=float(row["dx"]), dy=float(row["dy"]))

            # optional: Voids pro Patch (wenn Spalten vorhanden)
            if (
                "voids_csv" in df.columns
                and isinstance(row.get("voids_csv"), str)
                and row["voids_csv"].strip()
            ):
                voids_csv = row["voids_csv"].strip()
                polygons_csv = None
                if (
                    "polygons_csv" in df.columns
                    and isinstance(row.get("polygons_csv"), str)
                    and row["polygons_csv"].strip()
                ):
                    polygons_csv = row["polygons_csv"].strip()
                m.apply_voids_from_csv(voids_csv, polygons_csv=polygons_csv)

            meshes.append(m)

        # Merge: erstes Mesh ist Basis
        base = meshes[0]
        for m in meshes[1:]:
            base.merge_with_mesh(m, tol=tol)  # tol optional

        return base

    def Get_Meshing_Parameters(self):

        try:
            self.Meshing_Params = pd.read_csv("Meshing_Parameters/Meshing_Params.csv")
        except:
            self.Meshing_Params = pd.read_csv(
                "../Meshing_Parameters/Meshing_Params.csv"
            )

        _num_x = self.Meshing_Params.loc[
            self.Meshing_Params["Param"] == "num_x", "Value"
        ].values[0]
        _num_y = self.Meshing_Params.loc[
            self.Meshing_Params["Param"] == "num_y", "Value"
        ].values[0]
        _d_1 = self.Meshing_Params.loc[
            self.Meshing_Params["Param"] == "d1", "Value"
        ].values[0]
        _d_2 = self.Meshing_Params.loc[
            self.Meshing_Params["Param"] == "d2", "Value"
        ].values[0]

        if self.Meshing_Mode == "Meshing_R":
            self.Meshing = MeshingClass(
                d1=_d_1, d2=_d_2, num_x=int(_num_x), num_y=int(_num_y), elem_type="rect"
            )
            self.Meshing.generating_rectangular_mesh()

        elif self.Meshing_Mode == "Meshing_RQ":
            self.Meshing = MeshingClass(
                d1=0, d2=0, num_x=int(_num_x), num_y=int(_num_y), elem_type="rect"
            )
            self.Meshing.generating_quadrilateral_mesh_from_csv(
                "Meshing_Parameters/Meshing_Params_q.csv"
            )

        elif self.Meshing_Mode == "Meshing_MP":
            # Multi-Patch: alles aus eigener CSV
            self.Meshing = self.BuildMultiPatchMeshFromCSV(
                "Meshing_Parameters/Meshing_Patches.csv"
            )

        else:
            raise ValueError(f"Unknown Meshing_Mode: {self.Meshing_Mode}")

    def Get_Material_Parameters(self):
        try:
            self.Material_Params = pd.read_csv("Material_Parameters/Material.csv")
        except:
            self.Material_Params = pd.read_csv("../Material_Parameters/Material.csv")
        self.E = self.Material_Params.loc[
            self.Material_Params["Parameter"] == "E", "Value"
        ].values[0]
        self.nu = self.Material_Params.loc[
            self.Material_Params["Parameter"] == "nu", "Value"
        ].values[0]
        self.t = self.Material_Params.loc[
            self.Material_Params["Parameter"] == "t", "Value"
        ].values[0]
        self.gamma = self.Material_Params.loc[
            self.Material_Params["Parameter"] == "gamma", "Value"
        ].values[0]

    def Get_Element_Matrices(self):
        self.EL_Matrices = np.zeros((len(self.Meshing.EL), 8, 8))
        self.EL_AreaLoads = np.zeros((8, len(self.Meshing.EL)))
        self.EL_AreaLoads_Nodes = np.zeros((len(self.Meshing.EL), 4))

        for i in range(0, len(self.Meshing.EL), 1):
            coords = np.zeros((4, 2))
            for j in range(0, len(self.Meshing.EL[0]), 1):
                index = int(self.Meshing.EL[i][j])
                _x = self.Meshing.NL[index - 1][0]
                _y = self.Meshing.NL[index - 1][1]
                coords[j][0] = _x
                coords[j][1] = _y

            self.stiffness_matrix = Stiffness_Matrix(
                coords, self.E, self.nu, self.t, self.gamma, self.Meshing
            )
            self.stiffness_matrix.stiffness()

            self.EL_Matrices[i] = self.stiffness_matrix.K

            self.stiffness_matrix.element_load_vector()
            self.EL_AreaLoads[:, i] = self.stiffness_matrix.element_loading_vec
            self.EL_AreaLoads_Nodes[i, :] = self.Meshing.EL[i]

    def Assemble_Stiffness_Matrices(self):
        self.AssembleMatrix = Assembled_Matrices(
            self.Meshing,
            self.Meshing.EL,
            self.Meshing.NL,
            self.EL_Matrices,
            self.stiffness_matrix.node_loading_vec,
            self.stiffness_matrix.node_store_vec,
            self.EL_AreaLoads,
            self.EL_AreaLoads_Nodes,
        )
        self.AssembleMatrix.assemble_K()
        self.AssembleMatrix.Load_BC()
        self.AssembleMatrix.LoadInput()
        self.AssembleMatrix.apply_BC()
        self.AssembleMatrix.GenerateLoadVector()

    def SolveAssembled_Matrix(self):
        self.AssembleMatrix.Solve()

    def StoreElementDisplacements(self):
        self.AssembleMatrix.StoreElementDisplacements()

    def Ns_Mat(self, xi, eta):
        self.Ns_Matrix = np.zeros(
            (self.stiffness_matrix._PD, self.stiffness_matrix._NPE)
        )
        self.Ns_Matrix_T = np.zeros(
            (self.stiffness_matrix._NPE, self.stiffness_matrix._PD)
        )

        if self.stiffness_matrix._NPE == 3:
            self.Ns_Matrix[0][0] = 1
            self.Ns_Matrix[0][1] = 0
            self.Ns_Matrix[0][2] = -1

            self.Ns_Matrix[1][0] = 0
            self.Ns_Matrix[1][1] = 1
            self.Ns_Matrix[1][2] = -1

        if self.stiffness_matrix._NPE == 4:
            self.Ns_Matrix[0][0] = -0.25 * (1 - eta)
            self.Ns_Matrix[0][1] = 0.25 * (1 - eta)
            self.Ns_Matrix[0][2] = 0.25 * (1 + eta)
            self.Ns_Matrix[0][3] = -0.25 * (1 + eta)

            self.Ns_Matrix[1][0] = -0.25 * (1 - xi)
            self.Ns_Matrix[1][1] = -0.25 * (1 + xi)
            self.Ns_Matrix[1][2] = 0.25 * (1 + xi)
            self.Ns_Matrix[1][3] = 0.25 * (1 - xi)

    def E_Mat(self):
        """
        Function to calculate the material matrix for each element \n
        """
        self.E_el = np.zeros((3, 3))

        self.E_el[0][0] = 1
        self.E_el[0][1] = self.nu
        self.E_el[0][2] = 0

        self.E_el[1][0] = self.nu
        self.E_el[1][1] = 1
        self.E_el[1][2] = 0

        self.E_el[2][0] = 0
        self.E_el[2][1] = 0
        self.E_el[2][2] = (1 - self.nu) / 2

        self.E_el *= self.E / (1 - self.nu**2)

    def _q4_dN_dxi_eta(xi, eta):
        # Q4, Knotenreihenfolge: 1(-1,-1), 2(+1,-1), 3(+1,+1), 4(-1,+1)
        dN_dxi = 0.25 * np.array(
            [-(1 - eta), +(1 - eta), +(1 + eta), -(1 + eta)], dtype=float
        )

        dN_deta = 0.25 * np.array(
            [-(1 - xi), -(1 + xi), +(1 + xi), +(1 - xi)], dtype=float
        )

        return dN_dxi, dN_deta

    def _D_plane_stress(E, nu):
        c = E / (1.0 - nu**2)
        return c * np.array(
            [[1.0, nu, 0.0], [nu, 1.0, 0.0], [0.0, 0.0, (1.0 - nu) / 2.0]], dtype=float
        )

    def CalculateInnerElementForces_Gauss(self, compute_nodal=True, compute_principal=True):
        """
        Rückrechnung an 2x2 Gauss-Punkten je Q4-Element (plane stress).
        + optional:
        - Extrapolation GP -> Elementknoten -> globale Knotenwerte
        - Bestimmung globaler Knoten-Extrema
        - Hauptmembrankräfte (principal membrane forces)
        """

        import numpy as np

        # -------------------------
        # Gauss-Punkte 2x2
        # -------------------------
        gp = 1.0 / np.sqrt(3.0)
        gauss_pts = [(-gp, -gp), (+gp, -gp), (+gp, +gp), (-gp, +gp)]

        def dN_dxi_eta(xi, eta):
            dN_dxi = 0.25 * np.array([-(1 - eta), +(1 - eta), +(1 + eta), -(1 + eta)], dtype=float)
            dN_deta = 0.25 * np.array([-(1 - xi), -(1 + xi), +(1 + xi), +(1 - xi)], dtype=float)
            return dN_dxi, dN_deta

        def Nvals(xi, eta):
            return 0.25 * np.array([
                (1 - xi) * (1 - eta),
                (1 + xi) * (1 - eta),
                (1 + xi) * (1 + eta),
                (1 - xi) * (1 + eta),
            ], dtype=float)

        def D_plane_stress(E, nu):
            c = E / (1.0 - nu**2)
            return c * np.array([
                [1.0, nu, 0.0],
                [nu, 1.0, 0.0],
                [0.0, 0.0, (1.0 - nu) / 2.0],
            ], dtype=float)

        # -------------------------
        # Speicher
        # -------------------------
        n_elem = len(self.Meshing.EL)

        self.gp_xy     = np.zeros((n_elem, 4, 2), dtype=float)
        self.strain_gp = np.zeros((n_elem, 4, 3), dtype=float)
        self.stress_gp = np.zeros((n_elem, 4, 3), dtype=float)
        self.n_gp      = np.zeros((n_elem, 4, 3), dtype=float)

        # -------------------------
        # Material / Dicke
        # -------------------------
        E0  = float(getattr(self, "E", 210e9))
        nu0 = float(getattr(self, "nu", 0.3))
        t0  = float(getattr(self, "t", 1.0))

        # -------------------------
        # Loop über Elemente
        # -------------------------
        for e in range(n_elem):
            el_nodes = self.Meshing.EL[e]  # [n1,n2,n3,n4] (1-basiert)
            coords = np.array([self.Meshing.NL[nid - 1] for nid in el_nodes], dtype=float)  # (4,2)

            ue = np.asarray(self.AssembleMatrix.disp_element_matrix[:, e], dtype=float).reshape(8)

            E, nu, t = E0, nu0, t0
            D = D_plane_stress(E, nu)

            for k, (xi, eta) in enumerate(gauss_pts):
                dN_dxi, dN_deta = dN_dxi_eta(xi, eta)

                J = np.zeros((2, 2), dtype=float)
                J[0, 0] = np.dot(dN_dxi,  coords[:, 0])
                J[0, 1] = np.dot(dN_deta, coords[:, 0])
                J[1, 0] = np.dot(dN_dxi,  coords[:, 1])
                J[1, 1] = np.dot(dN_deta, coords[:, 1])

                detJ = np.linalg.det(J)
                if detJ <= 0:
                    raise ValueError(f"detJ <= 0 in element {e+1}. Check node ordering / mesh.")

                invJ = np.linalg.inv(J)

                dN_dx = invJ[0, 0] * dN_dxi + invJ[0, 1] * dN_deta
                dN_dy = invJ[1, 0] * dN_dxi + invJ[1, 1] * dN_deta

                B = np.zeros((3, 8), dtype=float)
                for i in range(4):
                    B[0, 2*i + 0] = dN_dx[i]
                    B[1, 2*i + 1] = dN_dy[i]
                    B[2, 2*i + 0] = dN_dy[i]
                    B[2, 2*i + 1] = dN_dx[i]

                eps = B @ ue
                sig = D @ eps

                N = Nvals(xi, eta)
                x_gp = float(np.dot(N, coords[:, 0]))
                y_gp = float(np.dot(N, coords[:, 1]))

                self.gp_xy[e, k, :]     = [x_gp, y_gp]
                self.strain_gp[e, k, :] = eps
                self.stress_gp[e, k, :] = sig
                self.n_gp[e, k, :]      = sig * t  # [n_x, n_y, n_xy]

        # Element-Mittelwerte
        self.stress_elem_avg = np.mean(self.stress_gp, axis=1)  # (n_elem,3)
        self.n_elem_avg      = np.mean(self.n_gp, axis=1)       # (n_elem,3)

        # ==========================================================
        # (1) Knotenwerte (Extrapolation GP -> Elementknoten -> global)
        # ==========================================================
        if compute_nodal:
            n_nodes = self.Meshing.NL.shape[0]

            # Extrapolationsmatrix für GP-Reihenfolge:
            # (-g,-g),(+g,-g),(+g,+g),(-g,+g) -> Knoten (-1,-1),(+1,-1),(+1,+1),(-1,+1)
            s2 = np.sqrt(3.0) / 2.0
            Mext = np.array([
                [1.0 + s2, -0.5,      1.0 - s2, -0.5     ],
                [-0.5,     1.0 + s2,  -0.5,      1.0 - s2],
                [1.0 - s2, -0.5,      1.0 + s2, -0.5     ],
                [-0.5,     1.0 - s2,  -0.5,      1.0 + s2],
            ], dtype=float)

            # element-knotenwerte
            self.stress_node_elem = np.zeros((n_elem, 4, 3), dtype=float)
            self.n_node_elem      = np.zeros((n_elem, 4, 3), dtype=float)

            for e in range(n_elem):
                self.stress_node_elem[e, :, :] = Mext @ self.stress_gp[e, :, :]
                self.n_node_elem[e, :, :]      = Mext @ self.n_gp[e, :, :]

            # globale knotenwerte via Mittelung
            sum_stress = np.zeros((n_nodes, 3), dtype=float)
            sum_n      = np.zeros((n_nodes, 3), dtype=float)
            cnt        = np.zeros((n_nodes,), dtype=int)

            for e in range(n_elem):
                el_nodes = self.Meshing.EL[e]  # 1-basiert
                for a in range(4):
                    nid0 = int(el_nodes[a]) - 1
                    sum_stress[nid0, :] += self.stress_node_elem[e, a, :]
                    sum_n[nid0, :]      += self.n_node_elem[e, a, :]
                    cnt[nid0]           += 1

            cnt_safe = np.maximum(cnt, 1)[:, None]
            self.stress_node = sum_stress / cnt_safe
            self.n_node      = sum_n / cnt_safe
            self.node_contrib_count = cnt

            # ---- Knoten-Extrema (max/min pro Komponente) ----
            def _extrema_dict(vec, name):
                NL = np.asarray(self.Meshing.NL, float)
                imax = int(np.nanargmax(vec))
                imin = int(np.nanargmin(vec))
                return {
                    "name": name,
                    "max": {"value": float(vec[imax]), "node": imax+1, "xy": (float(NL[imax,0]), float(NL[imax,1]))},
                    "min": {"value": float(vec[imin]), "node": imin+1, "xy": (float(NL[imin,0]), float(NL[imin,1]))},
                }

            self.extrema_nodes = {
                "sigma_x": _extrema_dict(self.stress_node[:, 0], "sigma_x"),
                "sigma_y": _extrema_dict(self.stress_node[:, 1], "sigma_y"),
                "tau_xy":  _extrema_dict(self.stress_node[:, 2], "tau_xy"),
                "n_x":     _extrema_dict(self.n_node[:, 0],      "n_x"),
                "n_y":     _extrema_dict(self.n_node[:, 1],      "n_y"),
                "n_xy":    _extrema_dict(self.n_node[:, 2],      "n_xy"),
            }

        # ==========================================================
        # (2) Hauptmembrankräfte (principal membrane forces)
        # ==========================================================
        if compute_principal:
            def principal_2d(nx, ny, nxy):
                """
                Rückgabe:
                n1, n2: Hauptmembrankräfte
                theta: Winkel der 1. Hauptachse gegen +x (rad)
                """
                n_avg = 0.5 * (nx + ny)
                R = np.sqrt((0.5 * (nx - ny))**2 + nxy**2)
                n1 = n_avg + R
                n2 = n_avg - R
                theta = 0.5 * np.arctan2(2.0 * nxy, (nx - ny))
                return n1, n2, theta

            # GP principal
            self.n_princ_gp = np.zeros((n_elem, 4, 3), dtype=float)  # [n1,n2,theta]
            for e in range(n_elem):
                for k in range(4):
                    nx, ny, nxy = self.n_gp[e, k, :]
                    n1, n2, th = principal_2d(nx, ny, nxy)
                    self.n_princ_gp[e, k, :] = [n1, n2, th]

            # element-avg principal (aus element-avg n)
            self.n_princ_elem_avg = np.zeros((n_elem, 3), dtype=float)
            for e in range(n_elem):
                nx, ny, nxy = self.n_elem_avg[e, :]
                n1, n2, th = principal_2d(nx, ny, nxy)
                self.n_princ_elem_avg[e, :] = [n1, n2, th]

            # nodal principal (wenn nodal vorhanden)
            if compute_nodal:
                self.n_princ_node = np.zeros((self.n_node.shape[0], 3), dtype=float)
                for i in range(self.n_node.shape[0]):
                    nx, ny, nxy = self.n_node[i, :]
                    n1, n2, th = principal_2d(nx, ny, nxy)
                    self.n_princ_node[i, :] = [n1, n2, th]

    def PlotStressAlongCut(self, cut_position, cut_direction="x", field="sigma_x"):
        import numpy as np
        import matplotlib.pyplot as plt
        from scipy.interpolate import griddata

        if not hasattr(self, "stress_elem_avg"):
            self.CalculateInnerElementForces_Gauss()

        field_idx = {"sigma_x": 0, "sigma_y": 1, "tau_xy": 2}
        j = field_idx[field]

        # Elementzentren
        centroids = []
        vals = []
        for e, el in enumerate(self.Meshing.EL):
            coords = np.array([self.Meshing.NL[nid - 1] for nid in el], dtype=float)
            centroids.append(coords.mean(axis=0))
            vals.append(float(self.stress_elem_avg[e, j]))
        centroids = np.array(centroids, dtype=float)  # (n_elem,2)
        vals = np.array(vals, dtype=float)  # (n_elem,)

        # Grid für Interpolation
        grid_x, grid_y = np.mgrid[
            centroids[:, 0].min() : centroids[:, 0].max() : 200j,
            centroids[:, 1].min() : centroids[:, 1].max() : 200j,
        ]

        grid_z = griddata(centroids, vals, (grid_x, grid_y), method="linear")

        if cut_direction == "x":
            cut_index = int(np.argmin(np.abs(grid_x[:, 0] - cut_position)))
            cut_vals = grid_z[cut_index, :]
            cut_coords = grid_y[cut_index, :]
            xlabel = "y"
            title = f"{field} along x={cut_position}"
        else:
            cut_index = int(np.argmin(np.abs(grid_y[0, :] - cut_position)))
            cut_vals = grid_z[:, cut_index]
            cut_coords = grid_x[:, cut_index]
            xlabel = "x"
            title = f"{field} along y={cut_position}"

        plt.figure()
        plt.plot(cut_coords, cut_vals, label=title)
        plt.xlabel(xlabel)
        plt.ylabel(field)
        plt.grid(True)
        plt.legend()
        plt.title(title)
        plt.show()



try:
    from .Iteration import IterationClass
    from .Plotting import PlottingStructure
    from .Iteration import IterationClass
    from .Plotting import PlottingStructure
except:
    from KIB_LAP.FACHWERKEBEN.Iteration import IterationClass
    from KIB_LAP.FACHWERKEBEN.Plotting import PlottingStructure 
    from KIB_LAP.FACHWERKEBEN.Iteration import IterationClass
    from KIB_LAP.FACHWERKEBEN.Plotting import PlottingStructure



import numpy as np
import pandas as pd
from tabulate import *


class Linear_Algebra_Class:
    def matrix_transpose_symmetric(self, matrix, result=None):
        return np.transpose(matrix)

    def matrix_multiplication(self, mat1, mat2):
        return np.matmul(mat1, mat2)


class Gaucho_Method:
    def __init__(self, J):
        self.J = J
        self.det_J = None
        self.J_inv = None

    def ldl_decomposition(self):
        self.L = np.linalg.cholesky(self.J)
        self.D = np.diag(np.diag(self.L))
        self.L = self.L / np.sqrt(np.diag(self.L))[:, None]

    def calculate_inverse_gaucho(self):
        self.J_inv = np.linalg.inv(self.J)
        return self.J_inv

    def calculate_determinant_gaucho(self):
        self.det_J = np.linalg.det(self.J)
        return self.det_J


class Stiffness_Matrix:
    def __init__(self, coords, E, nu, t, gamma, _mesh=None):
        self._coords = np.array(coords)
        self._E = E
        self._nu = nu
        self._t = t
        self._gamma = gamma

        self._coords_T = np.transpose(self._coords)
        self.K = np.zeros((len(coords), len(coords)))
        self.K_ij = np.zeros((len(coords[0]), len(coords[0])))
        self.J = np.zeros((len(coords[0]), len(coords[0])))
        self.J_inv = np.zeros((len(coords[0]), len(coords[0])))

        self.grad = np.zeros((len(coords[0]), len(coords)))
        self.result_grad = np.zeros((len(coords[0]), len(coords)))
        self.result_grad_T = np.zeros((len(coords), len(coords[0])))

        self._NPE = len(coords)
        self._PD = len(coords[0])
        self._GPE = 4  # 4 GP for rectangular, 3 for triangular

        # Gaussian integration
        self.xi, self.eta, self.alpha = 0, 0, 0

        # Material parameters
        self.delta = 0
        self.C = 0
        self.det_J = 0

        # Friends
        self.lin_al = Linear_Algebra_Class()
        self.gaucho_ptr = None

        self.mesh_data = _mesh

    def gauss_points(self, gp):
        sqrt3_inv = 1 / np.sqrt(3)
        gauss_points = [
            (-sqrt3_inv, -sqrt3_inv, 1),
            (sqrt3_inv, -sqrt3_inv, 1),
            (sqrt3_inv, sqrt3_inv, 1),
            (-sqrt3_inv, sqrt3_inv, 1),
        ]
        if self._GPE == 4:
            self.xi, self.eta, self.alpha = gauss_points[gp - 1]

    def stiffness(self):
        self.K = np.zeros((self._NPE * self._PD, self._NPE * self._PD))
        self.det_J_store = np.zeros((2, 2))

        self._coords_T = self.lin_al.matrix_transpose_symmetric(self._coords)

        for i in range(self._NPE):
            for j in range(self._NPE):
                for gp in range(1, self._GPE + 1):
                    self.gauss_points(gp)
                    self.grad_N_nat()
                    self.J = self.lin_al.matrix_multiplication(
                        self._coords_T, self.result_grad_T
                    )
                    self.gaucho_ptr = Gaucho_Method(self.J)

                    self.gaucho_ptr.ldl_decomposition()
                    self.J_inv = self.gaucho_ptr.calculate_inverse_gaucho()
                    self.det_J = self.gaucho_ptr.calculate_determinant_gaucho()

                    self.grad = self.lin_al.matrix_multiplication(  # Transformed B-Matrix (Isoparametric)
                        self.J_inv, self.result_grad
                    )

                    for a in range(self._PD):
                        for b in range(self._PD):
                            for c in range(self._PD):
                                for d in range(self._PD):
                                    self.K[a * self._NPE + i, c * self._NPE + j] += (
                                        self.grad[b, i]
                                        * self.constitutive(a + 1, b + 1, c + 1, d + 1)
                                        * self.grad[d, j]
                                        * self.det_J
                                        * self.alpha
                                        * self._t
                                    )

    def grad_N_nat(self):
        self.result_grad = np.zeros((self._PD, self._NPE))
        self.result_grad_T = np.zeros((self._NPE, self._PD))

        if self._NPE == 3:
            self.result_grad[0][0] = 1
            self.result_grad[0][1] = 0
            self.result_grad[0][2] = -1

            self.result_grad[1][0] = 0
            self.result_grad[1][1] = 1
            self.result_grad[1][2] = -1

        if self._NPE == 4:
            self.result_grad[0][0] = -0.25 * (1 - self.eta)
            self.result_grad[0][1] = 0.25 * (1 - self.eta)
            self.result_grad[0][2] = 0.25 * (1 + self.eta)
            self.result_grad[0][3] = -0.25 * (1 + self.eta)

            self.result_grad[1][0] = -0.25 * (1 - self.xi)
            self.result_grad[1][1] = -0.25 * (1 + self.xi)
            self.result_grad[1][2] = 0.25 * (1 + self.xi)
            self.result_grad[1][3] = 0.25 * (1 - self.xi)

        self.result_grad_T = self.lin_al.matrix_transpose_symmetric(self.result_grad)

    def N_nat(self, xi, eta):
        self.result_N = np.zeros((2, 8))
        self.result_N_T = np.zeros((8, 2))

        if self._NPE == 4:
            self.result_N[0][0] = 0.25 * (1 - eta) * (1 - xi)
            self.result_N[0][2] = 0.25 * (1 - eta) * (1 + xi)
            self.result_N[0][4] = 0.25 * (1 + eta) * (1 + xi)
            self.result_N[0][6] = 0.25 * (1 + eta) * (1 - xi)

            self.result_N[1][1] = 0.25 * (1 - eta) * (1 - xi)
            self.result_N[1][3] = 0.25 * (1 - eta) * (1 + xi)
            self.result_N[1][5] = 0.25 * (1 + eta) * (1 + xi)
            self.result_N[1][7] = 0.25 * (1 + eta) * (1 - xi)

        self.result_N_T = self.lin_al.matrix_transpose_symmetric(self.result_N)

    def delta_func(self, i, j):
        return 1 if i == j else 0

    def constitutive(self, i, j, k, l):
        return (self._E / (2 * (1 + self._nu))) * (
            self.delta_func(i, l) * self.delta_func(j, k)
            + self.delta_func(i, k) * self.delta_func(j, l)
        ) + (self._E * self._nu) / (1 - self._nu**2) * self.delta_func(
            i, j
        ) * self.delta_func(
            k, l
        )

    def element_load_vector(self):
        try:
            self.loading_table = pd.DataFrame(pd.read_csv("Loading/Element_Loads.csv"))
        except:
            self.loading_table = pd.DataFrame(
                pd.read_csv("../Loading/Element_Loads.csv")
            )

        self.element_loading_vec = np.zeros(8)

        for i in self.loading_table["type"]:
            if i == "top":
                self.loading_nodes = self.mesh_data.get_top_border_nodes()
                self.node_loading_vec = np.zeros(
                    (2, int(len(self.loading_nodes) - 1) * 2)
                )
                self.node_store_vec = np.zeros(int((len(self.loading_nodes) - 1) * 2))

                for p in range(0, len(self.loading_nodes) - 1):
                    npa = self.loading_nodes[p]
                    npe = self.loading_nodes[p + 1]

                    self.node_store_vec[p * 2] = npa
                    self.node_store_vec[p * 2 + 1] = npe

                    x_npa = self.mesh_data.get_coordinate_x(npa)
                    z_npa = self.mesh_data.get_coordinate_z(npa)
                    x_npe = self.mesh_data.get_coordinate_x(npe)
                    z_npe = self.mesh_data.get_coordinate_z(npa)

                    l_x = abs(x_npe - x_npa)
                    l_z = abs(z_npe - z_npa)

                    px = self.loading_table[self.loading_table["type"] == "top"][
                        "px"
                    ].values
                    pz = self.loading_table[self.loading_table["type"] == "top"][
                        "pz"
                    ].values

                    # Numerical integration of the load vector at each node with an loadings

                    eta = 1  # At the top boundary, the eta value is 1 for all loops
                    WP = 1  # Numerical integration with 2 Gauss Points
                    for x in range(0, 2, 1):
                        xi = -1 / np.sqrt(3) + 1 / np.sqrt(3) * 2 * x
                        self.N_nat(xi, eta)

                        self.p_a_x = (
                            self.result_N[0][6] * px
                        )  # The beginning node is always form-function N4
                        self.p_a_z = self.result_N[1][7] * pz
                        self.p_e_x = self.result_N[0][4] * px
                        self.p_e_z = self.result_N[1][5] * pz

                        self.node_loading_vec[0][p * 2] += self.p_a_x * l_x / 2
                        self.node_loading_vec[1][p * 2] += self.p_a_z * l_x / 2

                        self.node_loading_vec[0][p * 2 + 1] += self.p_e_x * l_x / 2
                        self.node_loading_vec[1][p * 2 + 1] += self.p_e_z * l_x / 2
            elif i == "bottom":
                self.loading_nodes = self.mesh_data.get_bottom_border_nodes()
                self.node_loading_vec = np.zeros(
                    (2, int(len(self.loading_nodes) - 1) * 2)
                )
                self.node_store_vec = np.zeros(int((len(self.loading_nodes) - 1) * 2))

                for p in range(0, len(self.loading_nodes) - 1):
                    npa = self.loading_nodes[p]
                    npe = self.loading_nodes[p + 1]

                    self.node_store_vec[p * 2] = npa
                    self.node_store_vec[p * 2 + 1] = npe

                    x_npa = self.mesh_data.get_coordinate_x(npa)
                    z_npa = self.mesh_data.get_coordinate_z(npa)
                    x_npe = self.mesh_data.get_coordinate_x(npe)
                    z_npe = self.mesh_data.get_coordinate_z(npa)

                    l_x = abs(x_npe - x_npa)
                    l_z = abs(z_npe - z_npa)

                    px = self.loading_table[self.loading_table["type"] == "bottom"][
                        "px"
                    ].values
                    pz = self.loading_table[self.loading_table["type"] == "bottom"][
                        "pz"
                    ].values

                    # Numerical integration of the load vector at each node with an loadings

                    eta = 1  # At the top boundary, the eta value is 1 for all loops
                    WP = 1  # Numerical integration with 2 Gauss Points
                    for x in range(0, 2, 1):
                        xi = -1 / np.sqrt(3) + 1 / np.sqrt(3) * 2 * x
                        self.N_nat(xi, eta)

                        self.p_a_x = (
                            self.result_N[0][6] * px
                        )  # The beginning node is always form-function N4
                        self.p_a_z = self.result_N[1][7] * pz
                        self.p_e_x = self.result_N[0][4] * px
                        self.p_e_z = self.result_N[1][5] * pz

                        self.node_loading_vec[0][p * 2] += self.p_a_x * l_x / 2
                        self.node_loading_vec[1][p * 2] += self.p_a_z * l_x / 2

                        self.node_loading_vec[0][p * 2 + 1] += self.p_e_x * l_x / 2
                        self.node_loading_vec[1][p * 2 + 1] += self.p_e_z * l_x / 2

           

            if i == "selfweight":
                """_summary_

                Note on the Jacobian: The Determinant is needed before the element load vector \n
                from self weight can be calculated. Because it remains constant for all calculations, there is not need to calculate it \n
                in each iteration \n
                """
                self.q_self = np.zeros(8)
                self.q_self[1:9:2] = self._gamma * self._t

                WP_xi = 1  # Numerical integration with 2 Gauss Points
                WP_eta = 1  # Numerical integration with 2 Gauss Points

                for x in range(0, 2, 1):
                    xi = -1 / np.sqrt(3) + 1 / np.sqrt(3) * 2 * x
                    for y in range(0, 2, 1):
                        eta = -1 / np.sqrt(3) + 1 / np.sqrt(3) * 2 * y

                        self.N_nat(xi, eta)

                        self.element_loading_vec[1] += (
                            self.q_self[1]
                            * self.result_N[0][0]
                            * self.det_J
                            * WP_xi
                            * WP_eta
                        )
                        self.element_loading_vec[3] += (
                            self.q_self[3]
                            * self.result_N[0][2]
                            * self.det_J
                            * WP_xi
                            * WP_eta
                        )
                        self.element_loading_vec[5] += (
                            self.q_self[5]
                            * self.result_N[0][4]
                            * self.det_J
                            * WP_xi
                            * WP_eta
                        )
                        self.element_loading_vec[7] += (
                            self.q_self[7]
                            * self.result_N[0][6]
                            * self.det_J
                            * WP_xi
                            * WP_eta
                        )

    def __str__(self):
        output = "NODAL COORDINATES\n"
        output += "\n".join(
            "  ".join(str(self._coords_T[i][j]) for j in range(len(self._coords_T[0])))
            for i in range(len(self._coords_T))
        )

        output += "\ngrad\n"
        output += "\n".join(
            " ".join(str(self.grad[i][j]) for j in range(len(self.grad[0])))
            for i in range(len(self.grad))
        )

        output += "\nResult grad T\n"
        output += "\n".join(
            " ".join(
                str(self.result_grad_T[i][j]) for j in range(len(self.result_grad_T[0]))
            )
            for i in range(len(self.result_grad_T))
        )

        output += "\nStiffness matrix K\n"
        output += "\n".join(
            " ".join(str(self.K[i][j]) for j in range(len(self.K[0])))
            for i in range(len(self.K))
        )

        return output




import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy.interpolate import interp1d

try:
    from Materialkennwerte_Beton import Baustoffe
    from Querschnittsbreite import PolygonSection
except:
    from KIB_LAP.Betonbau.Materialkennwerte_Beton import Baustoffe
    from KIB_LAP.Betonbau.Querschnittsbreite import PolygonSection


class Laengsbemessung:
    def __init__(
        self,
        fck=30,
        fyk=500,
        varphi_cal  = 0,
        epsilon_cs = 0,
        _LoadingType="csv",
        Iter="Bruch",
        Reinforcement="csv",
        Querschnittsbemessung="Polygon",
        P_m_inf = 0, A_p = 0, d_p1 = 0
    ):
        """_summary_

        Args:
            fck (int, optional): Charateristic compression strength of concrete material. Defaults to 30.
            fyk (int, optional): _description_. Defaults to 500.
            varphi_cal (int, optional): _description_. Defaults to 0.
            epsilon_cs (int, optional): _description_. Defaults to 0.
            _LoadingType (str, optional): _description_. Defaults to "csv".
            Iter (str, optional): _description_. Defaults to "Bruch".
            Reinforcement (str, optional): _description_. Defaults to "csv".
            Querschnittsbemessung (str, optional): _description_. Defaults to "Polygon".
            P_m_inf (int, optional): _description_. Defaults to 0.
            A_p (int, optional): _description_. Defaults to 0.
            d_p1 (int, optional): _description_. Defaults to 0.
        """
        # Calculation mode
        self.Iter = Iter
        # Material properties from input
        self.fck = fck
        self.fyk = fyk
        self.varepsilon_grenz = 2  # In Permille
        self.n_czone = 100
        # Init design values for loading
        self.LoadingType = _LoadingType

        self.NEd_GZT = 0
        self.MEd_GZT = 0
        self.MEds_GZT = 0
        self.Mrds_GZT = 0

        self.NEd_GZG = 0
        self.MEd_GZG = 0
        self.MEds_GZG = 0

        self.PM_inf = P_m_inf

        

        # Geometric parameters for the design
        self.Bewehrung = Reinforcement

        self.d_s1 = 0
        self.d_s2 = 0
        self.dp_1 = d_p1        # Abstand der Außenkante Biegezug bis elast. Zentrum des Spannglieds

        self.A_svorh = 0
        self.A_svorh2 = 0

        self.Ap = A_p
        try:
            self.epsilon_pm_inf = abs(self.PM_inf / self.Ap * 1/ self.Ep )    #Vordehnung
        except:
            self.epsilon_pm_inf = 0
        self.epsilon_yk = 1500/195000   # Für St 1550/1770


    def Baustoffe(self):
        BS = Baustoffe([self.fck,self.fyk])
        self.Ecm = BS.Ecm
        self.Es = BS.Es

        try:
            df = pd.read_csv("Materialparameter/Spannstahl.csv")
            self.Ep = float(df["Ep"])
        except:
            pass

    def Calculate_All(self):
        self.Querschnittswerte()
        self.Bewehrungsangaben()
        self.Baustoffe()
        self.ReadLoading()

        # if self.Iter == "Bruch":
        #     self.Iter_Compression()
        # else:
        #     self.Iter_Gebrauchslast()

    def Querschnittswerte(self, _height_test=0.5):
        # Example usage
        df = pd.read_csv("Polygon/Polygon.csv")
        self.vertices = df.values.tolist()

        self.polygon = PolygonSection(self.vertices)

        # Calculate the height of the polygon based on y-values

        y_values = [vertex[1] for vertex in self.vertices]
        self.height = abs(max(y_values) - min(y_values))
        self.z_su = self.height - self.polygon.centroid[0]
        self.z_so = self.height - self.z_su

        # Define the rotation angle
        self.angle_degrees = 0
        self.polygon.rotate(self.angle_degrees)

        height = _height_test
        section_width = self.polygon.calculate_section_width_at_height(height)

        return section_width

    def PlotCrossSection(self, _height_test=0.5):
        # Define the height at which to calculate the section width
        height = _height_test
        section_width = self.polygon.calculate_section_width_at_height(height)
        print(
            f"Section width at height {height} after rotating by {self.angle_degrees} degrees: {section_width}"
        )

        # Plot the polygon and the horizontal line
        self.polygon.plot(height)

    def Bewehrungsangaben(self):
        if self.Bewehrung == "csv":
            df = pd.read_csv("Bewehrung/Linienbewehrung.csv")
            print(df)
            for i in range(0, len(df["Lage"]), 1):
                Lage = df["Lage"][i]
                if Lage == "Unten":
                    self.d_s1 = (
                        df["dsl [m]"][i] * 0.5 + df["cnom [m]"][i] + df["dsw [m]"][i]
                    )
                    self.A_svorh = df["As [cm**2]"][i]
                elif Lage == "Oben":
                    self.d_s2 = (
                        df["dsl [m]"][i] * 0.5 + df["cnom [m]"][i] + df["dsw [m]"][i]
                    )
                    self.A_svorh2 = df["As [cm**2]"][i]

            self.z_ds1 = self.z_su - self.d_s1
            self.z_ds2 = self.z_so - self.d_s2

    def ReadLoading(self):
        if self.LoadingType == "csv":
            df = pd.read_csv("Lasteingabe/Lasten.csv")
            print(df)
            self.Zugseite = None
            for i in range(0, len(df["Grenzzustand"]), 1):
                if df["Grenzzustand"][i] == "GZT":
                    self.NEd_GZT = df["NEd in [MN]"][i] 
                    self.MEd_GZT = df["MEd in [MNm]"][i]
                elif df["Grenzzustand"][i] == "GZG":
                    self.NEd_GZG = df["NEd in [MN]"][i] 
                    self.MEd_GZG = df["MEd in [MNm]"][i]

            if self.MEd_GZT >= 0:
                self.MEds_GZT = abs(
                    self.MEd_GZT - self.NEd_GZT * self.z_ds1 + self.PM_inf * (self.dp_1 - self.d_s1)
                )  # Bezug auf die Biegezugbewehrung (Hier UNTEN)
                self.NEd_GZT -= self.PM_inf
                self.Zugseite_GZT = "UNTEN"
                self.d = self.height - self.d_s1
            elif self.MEd_GZT < 0:
                self.MEds_GZT = abs(
                    self.MEd_GZT + self.NEd_GZT * self.z_ds2 - self.PM_inf * (self.dp_1 - self.d_s1)
                )  # Bezug auf die Biegezugbewehrung (Hier OBEN)
                self.NEd_GZT -= self.PM_inf
                self.Zugseite_GZT = "OBEN"
                self.d = self.height - self.d_s2

            if self.MEd_GZG >= 0:
                self.MEds_GZG = self.MEd_GZG - self.NEd_GZG * self.z_ds1  + self.PM_inf * (self.dp_1 - self.d_s1)
                self.NEd_GZG -= self.PM_inf
                self.Zugseite_GZG = "UNTEN"
                self.d = self.height - self.d_s1
            elif self.MEd_GZG < 0:
                self.MEds_GZG = self.MEd_GZG + self.NEd_GZG * self.z_ds2  - self.PM_inf * (self.dp_1 - self.d_s1)
                self.NEd_GZG -= self.PM_inf
                self.Zugseite_GZG = "OBEN"
                self.d = self.height - self.d_s2

        else:
            self.NEd_GZT = float(
                input("Geben Sie die Normalkraft NEd im GZT in [MN] ein: \n")
            )
            self.MEd_GZT = float(
                input("Geben Sie das Biegemoment im GZT in [MN] ein: \n")
            )
            if self.MEd_GZT >= 0:
                self.MEds_GZT = self.MEd_GZT - self.NEd_GZT * self.z_ds1
            elif self.MEd_GZT < 0:
                self.MEds_GZT = self.MEd_GZT + self.NEd_GZT * self.z_ds1

            if self.MEd_GZG >= 0:
                self.MEds_GZG = self.MEd_GZG - self.NEd_GZG * self.z_ds1
            elif self.MEd_GZG < 0:
                self.MEds_GZG = self.MEd_GZG + self.NEd_GZG * self.z_ds1

        # Export loading parameters to the output folder
        self.zsi_GZT = None
        self.zsi_GZG = None

        if self.Zugseite_GZT == "UNTEN":
            self.zsi_GZT = self.z_ds1
        else:
            self.zsi_GZT = self.z_ds2

        if self.Zugseite_GZG == "UNTEN":
            self.zsi_GZG = self.z_ds1
        else:
            self.zsi_GZG = self.z_ds2

        df = pd.DataFrame(
            {
                "GZT": [
                    self.NEd_GZT,
                    self.MEd_GZT,
                    self.zsi_GZT,
                    self.MEds_GZT,
                    self.Zugseite_GZT,
                ],
                "GZG": [
                    self.NEd_GZG,
                    self.MEd_GZG,
                    self.zsi_GZG,
                    self.MEds_GZG,
                    self.Zugseite_GZG,
                ],
            },
            index=[
                "NEd [MN]",
                "MEd in [MNm]",
                "zsi in [m]",
                "|MEds| in [MNm]",
                "Zugseite",
            ],
        )

        df.to_csv("Output/Design_Forces.csv")

    def Sigma_ParabalRechteck(self, _varepsilon):
        if _varepsilon <= self.varepsilon_grenz:
            sigma = self.fcd * (1 - (1 - _varepsilon / self.varepsilon_grenz) ** 2)
            return sigma
        else:
            sigma = self.fcd
            return sigma

    def Sigma_Gebrauchslasten(self, _varepsilon):
        """
        This function returns the concrete stresses under
        servicability strains smaller than 0.5e-3.
        Args:
            _varepsilon (_type_): _description_

        Returns:
            _type_: _description_
        """
        return self.Ecm * _varepsilon * 1e-3

    def Iter_Compression(self):
        iter = 0
        self.epss = 25
        self.epsc = 0.00
        self.dimensionsless_moment = 0
        self.limit_dimensionless_moment = 0.296
        self.bcm_i = np.zeros(self.n_czone - 1)
        self.hcm_i = np.zeros(self.n_czone - 1)

        self.Fc_i = np.zeros(self.n_czone - 1)

        while self.Mrds <= self.Meds and iter <= 10000:
            self.xi = self.epsc / (self.epss + self.epsc)
            self.hu = self.height * (1 - self.xi)

            self.hc_i = np.linspace(self.hu, self.height, self.n_czone)
            self.Mrds = 0

            for i in range(0, self.n_czone - 1, 1):
                self.hcm_i[i] = 0.5 * (self.hc_i[i] + self.hc_i[i + 1])
                self.bcm_i[i] = 0.5 * (
                    self.Querschnittswerte(self.hc_i[i])
                    + self.Querschnittswerte(self.hc_i[i + 1])
                )

            for i in range(0, self.n_czone - 1, 1):
                epsilon_ci = abs(
                    self.epss
                    - (self.epsc + self.epss) / self.d * (self.hcm_i[i] - self.d_s1)
                )
                sigma_ci = self.Sigma_ParabalRechteck(epsilon_ci)

                self.Fc_i[i] = (
                    (self.hc_i[i + 1] - self.hc_i[i]) * self.bcm_i[i] * sigma_ci
                )

                self.Mrds += self.Fc_i[i] * (self.hcm_i[i] - self.d_s1)

            iter += 1

            if self.epsc >= 3.5:
                while self.Mrds <= self.Meds and iter <= 10000:
                    self.Mrds = 0

                    self.xi = self.epsc / (self.epss + self.epsc)
                    self.hu = self.height * (1 - self.xi)
                    self.hc_i = np.linspace(self.hu, self.height, self.n_czone)

                    for i in range(0, self.n_czone - 1, 1):
                        self.hcm_i[i] = 0.5 * (self.hc_i[i] + self.hc_i[i + 1])
                        self.bcm_i[i] = 0.5 * (
                            self.Querschnittswerte(self.hc_i[i])
                            + self.Querschnittswerte(self.hc_i[i + 1])
                        )
                        epsilon_ci = abs(
                            self.epss
                            - (self.epsc + self.epss)
                            / self.d
                            * (self.hcm_i[i] - self.d_s1)
                        )
                        sigma_ci = self.Sigma_ParabalRechteck(epsilon_ci)
                        self.Fc_i[i] = (
                            (self.hc_i[i + 1] - self.hc_i[i]) * self.bcm_i[i] * sigma_ci
                        )

                        self.Mrds += self.Fc_i[i] * (self.hcm_i[i] - self.d_s1)

                    iter += 1

                    if abs(self.Mrds - self.Meds) > 0.15:
                        self.epss -= 0.1
                    elif abs(self.Mrds - self.Meds) > 0.02:
                        self.epss -= 0.01
                    else:
                        self.epss -= 0.001

            if abs(self.Mrds - self.Meds) > 0.15:
                self.epsc += 0.1
            elif abs(self.Mrds - self.Meds) > 0.02:
                self.epsc += 0.01
            else:
                self.epsc += 0.0001

        self.F_sd = self.Fc_i.sum() + self.NEd
        self.A_serf = self.F_sd / self.fyd

        print(
            "The required reinforcement for bending is ",
            self.A_serf * 100**2,
            "cm**2",
        )
        
    def Iter_Gebrauchslast(self):
        iter = 0

        self.bcm_i = np.zeros(self.n_czone - 1)
        self.hcm_i = np.zeros(self.n_czone - 1)
        self.zcm_i = np.zeros(self.n_czone - 1)
        self.eps_zwischen = np.zeros(self.n_czone - 1)
        self.Fc_i = np.zeros(self.n_czone - 1)

        self.Fc_ges = 0
        self.F_c_list = []
        self.F_s1_ges = 0
        self.F_s1_list = []

        sum_h = []
        sum_F_s1 = []
        sum_F_s2 = []
        sum_F_c = []

        xi = 1e-4
        result = 1

        print("Iteration begins")
        p = 0
        while xi < 0.70:
            x = xi * self.d
            self.hu = self.height * (1 - xi)
            self.hc_i = np.linspace(self.hu, self.height, self.n_czone)

            for i in range(0, self.n_czone - 1, 1):
                self.hcm_i[i] = 0.5 * (self.hc_i[i] + self.hc_i[i + 1])
                self.bcm_i[i] = 0.5 * (
                    self.Querschnittswerte(self.hc_i[i])
                    + self.Querschnittswerte(self.hc_i[i + 1])
                )
                self.zcm_i[i] = self.d -x + x/self.n_czone * (i+1)

                self.eps_zwischen[i] = (i+1)/(self.n_czone+1) * self.bcm_i[i] * (self.hc_i[1] - self.hc_i[0]) * self.Ecm * self.zcm_i[i]
                

            epsilon_c2 = self.MEds_GZG / (
                self.eps_zwischen.sum()
                + self.Es
                * (1 - self.d_s2 / x)
                * (self.d - self.d_s2)
                * self.A_svorh2
                * 0.01**2
            )

            

            epsilon_s1 = epsilon_c2 * (self.d / x - 1)
            epsilon_s2 = epsilon_c2 * (1 - self.d_s2 / x)

            self.F_ci = 0
            self.epc_i= np.zeros(self.n_czone - 1)
            for i in range(0, self.n_czone-1, 1):
                self.epc_i[i] = epsilon_c2 * (i+1)/self.n_czone
                self.F_ci -= epsilon_c2 * (i+1)/(self.n_czone+1) * self.bcm_i[i] * (self.hc_i[1] - self.hc_i[0]) * self.Ecm

            if self.F_ci > 0:
                self.F_ci = 0

            self.F_s1 = (self.A_svorh * 0.01**2) * epsilon_s1 * self.Es
            self.F_s2 = epsilon_s2 * self.Es * self.A_svorh2 * 0.01**2 * (-1)

            result = - self.NEd_GZG + self.F_s1 + self.F_ci + self.F_s2

            sum_h.append(result)
            sum_F_s1.append(self.F_s1)
            sum_F_s2.append(self.F_s2)
            sum_F_c.append(self.F_ci)

            if abs(result) < 0.001:
                print("The iterated compression zone height xi is ", xi ,"and x = ", xi*self.d)
                self.xi = xi
                break
            if abs(result) > 0.5:
                xi += 0.01
            elif abs(result) > 0.01:
                xi += 0.001
            else:
                xi += 0.0001
            
       
            p+=1

        self.epsilon_c2_end = epsilon_c2

    def Iter_Gebrauchslast_Rect(self):
        iter = 0

        self.bcm_i = np.zeros(self.n_czone - 1)
        self.hcm_i = np.zeros(self.n_czone - 1)
        self.Fc_i = np.zeros(self.n_czone - 1)

        self.Fc_ges = 0
        self.F_c_list = []
        self.F_s1_ges = 0
        self.F_s1_list = []

        sum_h = []
        sum_F_s1 = []
        sum_F_s2 = []
        sum_F_c = []

        resu = []

        Flag_One = False
        Flag_Two = False
        Flag_Three = False

        b = self.Querschnittswerte(self.height / 2)
        xi = 1e-4
        result = 1

        print("Iteration begins")
        p = 0
        while xi < 0.70:
            x = xi * self.d

            epsilon_c2 = self.MEds_GZG / (
                (self.d - x / 3) * (0.5 * b * x * self.Ecm)
                + self.Es
                * (1 - self.d_s2 / x)
                * (self.d - self.d_s2)
                * self.A_svorh2
                * 0.01**2
            )

            sigma_c2 = epsilon_c2 * self.Ecm
            epsilon_s1 = epsilon_c2 * (self.d / x - 1)
            epsilon_s2 = epsilon_c2 * (1 - self.d_s2 / x)

            self.F_ci = 0.5 * b * x * sigma_c2 * (-1)
            if self.F_ci > 0:
                self.F_ci = 0

            self.F_s1 = (self.A_svorh * 0.01**2) * epsilon_s1 * self.Es
            self.F_s2 = epsilon_s2 * self.Es * self.A_svorh2 * 0.01**2 * (-1)

            result = - self.NEd_GZG + self.F_s1 + self.F_ci + self.F_s2

            sum_h.append(result)
            sum_F_s1.append(self.F_s1)
            sum_F_s2.append(self.F_s2)
            sum_F_c.append(self.F_ci)

            if abs(result) < 0.0001:
                print("The iterated compression zone height xi is ", xi ,"and x = ", xi*self.d)
                self.xi = xi
                break
            
            if abs(result) > 0.5:
                xi += 0.01
            elif abs(result) > 0.01:
                xi += 0.001
            else:
                xi += 0.0001
            
       
            p+=1
        self.epsilon_c2_end = epsilon_c2

    def Iter_Gebrauchslast_Spannbeton(self):
        iter = 0

        self.bcm_i = np.zeros(self.n_czone - 1)
        self.hcm_i = np.zeros(self.n_czone - 1)
        self.Fc_i = np.zeros(self.n_czone - 1)

        self.Fc_ges = 0
        self.F_c_list = []
        self.F_s1_ges = 0
        self.F_s1_list = []

        sum_h = []
        sum_F_s1 = []
        sum_F_s2 = []
        sum_F_c = []
        sum_F_p = []

        self.F_s2 = 0
        self.F_p = 0
        self.F_s1 = 0

        xi = 1e-5

        b = self.Querschnittswerte(self.height / 2)
        print("Iteration begins")

        while xi < 0.60:
            x = xi * self.d
            epsilon_c2 = self.MEds_GZG / (
                (self.d - x / 3) * (0.5 * b * x * self.Ecm)
                + self.F_s2 * (self.d - self.d_s2)
                - self.F_p * (self.d - (self.height - self.dp_1))
            )

            sigma_c2 = epsilon_c2 * self.Ecm
            epsilon_s1 = epsilon_c2 * (self.d / x - 1)
            epsilon_s2 = epsilon_c2 * (1 - self.d_s2 / x)

            epsilon_p = epsilon_c2 + (abs(epsilon_c2) + epsilon_s1)/self.d * (self.height - self.dp_1) # Additional strains in the prestressing cable

            self.F_ci = 0.5 * b * x * sigma_c2

            if self.F_ci < 0:
                self.F_ci = 0
            if (epsilon_s1 * self.Es <= self.fyk):
                self.F_s1 = (self.A_svorh * 0.01**2) * epsilon_s1 * self.Es
            else:
                self.F_s1 = (self.A_svorh * 0.01**2) * self.fyk
            if (epsilon_s2 * self.Es <= self.fyk):
                self.F_s2 = epsilon_s2 * self.Es * self.A_svorh2 * 0.01**2
            else:
                self.F_s2 = - self.fyk * self.A_svorh2 * 0.01**2

            if (abs(epsilon_p  +self.epsilon_pm_inf) <= self.epsilon_yk):
                self.F_p = abs((epsilon_p)) * self.Ep * self.Ap 
            else:
                self.F_p = 1500 * self.Ap 

            result = -self.NEd_GZG + self.F_s1 - self.F_ci + self.F_s2 + self.F_p

            sum_h.append(result)
            sum_F_s1.append(self.F_s1)
            sum_F_s2.append(self.F_s2)
            sum_F_c.append(self.F_ci)

            if abs(result) < 0.0001:
                print("The iterated compression zone height xi is ", xi)
                self.xi = xi
                break


            if abs(result) > 0.7:
                xi += 0.0001
            elif abs(result) > 0.10:
                xi += 0.00001
            else:
                xi += 0.000001

        print("Ned", self.NEd_GZG)
        print("xi",xi)
        print("Sum H", result)
        print("MEds - GZG", self.MEds_GZG)
        print("NEd - GZG", self.NEd_GZG)

        print("Fcd", self.F_ci)
        print("FP", self.F_p)
        print("Fs1" , self.F_s1)
        print("Fs2", self.F_s2)


# Laengs = Laengsbemessung(P_m_inf=0,A_p=81*0.01**2,d_p1=0.12)

# Laengs.Calculate_All()

# Laengs.Iter_Gebrauchslast()

# print("RESULTS ITERATION")
# print("Fcd" ,Laengs.F_ci)
# print("Fs1",Laengs.F_s1)
# print("Fs2",Laengs.F_s2)
# print("NEd",Laengs.NEd_GZG)

# print("eps2", Laengs.epsilon_c2_end)

# print("z_cm_i")
# plt.plot(Laengs.zcm_i)
# plt.show()

# print("Sigma_s1 [MPa]", Laengs.F_s1 / (Laengs.A_svorh*0.01**2))
# print("Sigma_s2 [MPa]", Laengs.F_s2 / (Laengs.A_svorh2*0.01**2))


# Laengs.Iter_Gebrauchslast_Rect()

# print("RESULTS - SEMI_ANALYTICAL")

# print("Fcd" ,Laengs.F_ci)
# print("Fs1",Laengs.F_s1)
# print("Fs2",Laengs.F_s2)
# print("NEd",Laengs.NEd_GZG)

# print("eps2", Laengs.epsilon_c2_end)

# print("Sigma_s1 [MPa]", Laengs.F_s1 / (Laengs.A_svorh*0.01**2))
# print("Sigma_s2 [MPa]", Laengs.F_s2 / (Laengs.A_svorh2*0.01**2))


# Laengs.Iter_Gebrauchslast_Spannbeton()

# print("Sigma_s1 [MPa]", Laengs.F_s1 / (Laengs.A_svorh*0.01**2))
# print("Sigma_s2 [MPa]", Laengs.F_s2 / (Laengs.A_svorh2*0.01**2))



# DEPENDENCIES
import copy  # Allows us to create copies of objects in memory
import math  # Math functionality
import numpy as np  # Numpy for working with arrays
import matplotlib.pyplot as plt  # Plotting functionality
import matplotlib.colors  # For colormap functionality
import ipywidgets as widgets
from glob import glob  # Allows check that file exists before import
from numpy import genfromtxt  # For importing structure data from csv
import pandas as pd

class Input:
    def __init__(self):
        print("Input-Class")
        self.NodalData()
        self.EdgeData()
        self.CableData()
        self.RestraintData()
        self.SpringData()
        self.ForceData()


    # =================================START OF DATA IMPORT================================
    def NodalData(self):
        # MANDATORY IMPORT: nodal coordinates
        if glob("data/Vertices.csv"):
            self.nodes = genfromtxt("data/Vertices.csv", delimiter=",")
            print("1. 🟢 Vertices.csv imported")
        else:
            print("1. 🛑 STOP: Vertices.csv not found")

    def EdgeData(self):
        # MANDATORY IMPORT: member definitions
        if glob("data/Edges.csv"):
            self.members = genfromtxt("data/Edges.csv", delimiter=",")
            self.members = np.int_(self.members)
            self.nDoF = (
                np.amax(self.members) * 2
            )  # Total number of degrees of freedom in the problem
            print("2. 🟢 Edges.csv imported")
        else:
            print("2. 🛑 STOP: Edges.csv not found")

    def RestraintData(self):
        # Prüfen, ob die Datei existiert
        if glob("data/Restraint-Data.csv"):
            # CSV einlesen; da es sich um eine einzelne Spalte handelt, reicht der Standard
            self.restraintData = genfromtxt("data/Restraint-Data.csv")
            # Sicherstellen, dass die Daten mindestens ein 1D-Array sind
            self.restraintData = np.atleast_1d(self.restraintData)
            # In Integer umwandeln (falls als float gelesen)
            self.restraintData = np.int_(self.restraintData)
            # Daten flach machen (bei einer einzelnen Spalte ist das optional)
            flatData = self.restraintData.flatten()
            # 0-Werte entfernen (0 = kein Einspannen)
            self.restrainedDoF = flatData[flatData != 0].tolist()
            # Von den in der CSV angegebenen Freiheitsgraden (beginnend bei 1)
            # zu Python-Index (beginnend bei 0) konvertieren
            self.restrainedIndex = [x - 1 for x in self.restrainedDoF]
            # Unbeschränkte Freiheitsgrade berechnen
            self.freeDoF = np.delete(np.arange(0, self.nDoF), self.restrainedIndex)
            print("3. 🟢 Restraint-Data.csv imported")
        else:
            print("3. 🛑 STOP: Restraint-Data.csv not found")

    def SpringData(self):
        if glob("data/springs.csv"):
            # Annahme: Die erste Zeile enthält Header
            df = pd.read_csv("data/springs.csv")
            # Numerische Spalten
            numeric_columns = ['no','Node', 'c_const[N/m]']
            df_numeric = df[numeric_columns]
            # Konvertiere das DataFrame zu einem NumPy-Array vom Typ int
            self.springLocationData = df_numeric.to_numpy(dtype=int)
            # Save the spring directions
            self.SpringDirections = df['Dir'].to_numpy()
            print("4. 🟢 springs.csv imported")
        else:
            self.forceLocationData = []
            print("4. ⚠️ springs.csv not found")

    def ForceData(self):
        # OPTIONAL IMPORT: force location data
        if glob("data/Force-Data.csv"):
            # Annahme: Die erste Zeile enthält Header
            df = pd.read_csv("data/Force-Data.csv")
            
            # Wähle nur die numerischen Spalten aus (z.B. 'Node' und 'P[N]')
            numeric_columns = ['Node', 'P[N]']
            df_numeric = df[numeric_columns]
            
            # Konvertiere das DataFrame zu einem NumPy-Array vom Typ int
            self.forceLocationData = df_numeric.to_numpy(dtype=int)
            
            # Bestimme die Anzahl der Dimensionen des Arrays
            self.nForces = self.forceLocationData.ndim
            
            # Falls das Array weniger als 2-dimensional ist, füge eine zusätzliche Dimension hinzu
            if self.nForces < 2:
                self.forceLocationData = np.expand_dims(self.forceLocationData, axis=0)

            # (Optional) Verarbeite die 'Dir'-Spalte, falls benötigt
            # Beispiel: Speichere die Richtungen separat
            self.forceDirections = df['Dir'].to_numpy()
            
            print("4. 🟢 Force-Data.csv imported")
        else:
            self.forceLocationData = []
            print("4. ⚠️ Force-Data.csv not found")

    def CableData(self):
        #MANDATORY IMPORT: cable definitions
        if glob('data/Cables.csv'): 
            self.cables = genfromtxt('data/Cables.csv', delimiter=',') 
            print('3. 🟢 Cables.csv imported')
        else: 
            self.cables = []
            print('3. 🛑 STOP: Cables.csv not found')


import numpy as np


class Output_Class:
    def __init__(self):
        init = None
    def report_nodal_extrema(self,fieldname):
        print("Max. x- Displacement: ", max(fieldname.AssembleMatrix.x_disp), "[m]")
        print("Min. x- Displacement: ", min(fieldname.AssembleMatrix.x_disp), "[m]")
        print("Max. z- Displacement: ", max(fieldname.AssembleMatrix.z_disp), "[m]")
        print("Min. z- Displacement: ", min(fieldname.AssembleMatrix.z_disp), "[m]")

    def report_extrema(self,field_name, arr_2d, gp_xy):
        """
        arr_2d: shape (n_elem, 4)  -> Werte je Element und Gauss-Punkt
        gp_xy : shape (n_elem, 4, 2) -> phys. Koordinaten der Gauss-Punkte
        """
        n_elem, n_gp = arr_2d.shape

        # MAX
        imax = np.argmax(arr_2d)
        emax, gmax = np.unravel_index(imax, arr_2d.shape)
        vmax = float(arr_2d[emax, gmax])
        x_max, y_max = gp_xy[emax, gmax, :]

        # MIN
        imin = np.argmin(arr_2d)
        emin, gmin = np.unravel_index(imin, arr_2d.shape)
        vmin = float(arr_2d[emin, gmin])
        x_min, y_min = gp_xy[emin, gmin, :]

        print(f"\n--- {field_name} ---")
        print(f"MAX: {vmax:.6g}  in Element {emax+1}, GP {gmax+1}  at (x,y)=({x_max:.6g},{y_max:.6g})")
        print(f"MIN: {vmin:.6g}  in Element {emin+1}, GP {gmin+1}  at (x,y)=({x_min:.6g},{y_min:.6g})")


import numpy as np
import matplotlib.pyplot as plt

class CircleSection:
    def __init__(self, radius):
        """
        Initializes the circle with the given radius.
        
        Parameters:
        radius (float): Radius of the circle.
        """
        self.radius = radius

    def calculate_section_width_at_height(self, height):
        """
        Calculates the width of the circle section at a specific height (y-value).
        
        Parameters:
        height (float): The y-value at which to calculate the section width.
        
        Returns:
        float: Width of the circle section at the specified height.
        """
        if abs(height) > self.radius:
            return 0.0
        return 2 * np.sqrt(self.radius**2 - height**2)

    def plot(self, height=None):
        """
        Plots the circle and optionally a horizontal line at the specified height.
        
        Parameters:
        height (float, optional): The y-value at which to plot a horizontal line.
        """
        theta = np.linspace(0, 2 * np.pi, 100)
        x = self.radius * np.cos(theta)
        y = self.radius * np.sin(theta)

        plt.figure()
        plt.plot(x, y, 'b-', label='Circle')
        plt.fill(x, y, 'b', alpha=0.3)
        if height is not None:
            plt.axhline(y=height, color='g', linestyle='--', label=f'Height = {height}')
            section_width = self.calculate_section_width_at_height(height)
            plt.plot([-section_width / 2, section_width / 2], [height, height], 'r-', linewidth=2, label=f'Section Width = {section_width:.2f}')
        plt.legend()
        plt.xlabel('x')
        plt.ylabel('y')
        plt.title('Circle and Section Width at Specific Height')
        plt.axis('equal')
        plt.show()

# Example usage
radius = 1.0
circle = CircleSection(radius)

# Define the height at which to calculate the section width
height = -0.5
section_width = circle.calculate_section_width_at_height(height)
print(f"Section width at height {height} is: {section_width}")

# Plot the circle and the horizontal line
circle.plot(height)



import numpy as np
from Stabberechnung_Klasse import StabRitz
from tabulate import tabulate
import matplotlib.pyplot as plt
import pandas as pd


class Biegedrillknicken:
    def __init__(self):
        print("Klasse")
        self.SG_TH_I = StabRitz(10, 0, 10, 1, 0, 0, 0, 0, 10, "-", "FE")
        self.SG_TH_I.Calculate_All()
        print("________Berechnung Biegedrillknicken_____________")
        # Cross Section Properties
        self.E = self.SG_TH_I.Querschnitt.E
        self.G = self.SG_TH_I.Querschnitt.G
        
        self.I_zz = self.SG_TH_I.Querschnitt.I_zz
        self.EIzz =  self.E * self.I_zz
        self.I_w = self.SG_TH_I.Querschnitt.I_w
        self.EIw =  self.E * self.I_w
        self.I_t = self.SG_TH_I.Querschnitt.I_T_GESAMT
        self.GIt = self.G * self.I_t
        self.z_M = self.SG_TH_I.Querschnitt.z_M  # Shear midpoint
        self.r = self.SG_TH_I.Querschnitt.ry

        self.z_j = 0#self.z_M - self.r / 2

        plt.show()

        print(self.EIw)
        print("IW", self.I_w)
        print(self.GIt)
        print("IT", self.I_t)
        print(self.EIzz)
        print("IZZ", self.I_zz)
        print("rz", self.r)

        # Inner Forces (TH.I.OG)
        self.My_x = self.SG_TH_I.m_element_inter  # Vektor mit Biegemoment
        self.l_x = self.SG_TH_I.l_x
        self.x = self.SG_TH_I.list_lx

        self.ne = self.SG_TH_I.reihen

        self.K_I_IW = np.zeros(
            (2 * (self.ne + 1), 2 * (self.ne + 1))
        )  # Steifigkeitsmatrizen mit konstanten Anteilen
        self.K_I_IT = np.zeros((2 * (self.ne + 1), 2 * (self.ne + 1)))
        self.K_I_q = np.zeros((2 * (self.ne + 1), 2 * (self.ne + 1)))
        self.K_I_P = np.zeros((2 * (self.ne + 1), 2 * (self.ne + 1)))

        self.K_I = np.zeros((2 * (self.ne + 1), 2 * (self.ne + 1)))

        self.K_II_Thet = np.zeros(
            (2 * (self.ne + 1), 2 * (self.ne + 1))
        )  # Steifigkeitsmatrix nach TH. II.OG für quadratischen Anteil
        self.K_II_Thet_s = np.zeros((2 * (self.ne + 1), 2 * (self.ne + 1)))

    def Calculate_All(self):
        self.Construct_IW_Mat()
        self.Construct_GIT_Mat()
        self.Construct_Mcr_Mat_1()
        self.Construct_Mcr_Mat_2()
        self.calculate_alpha_crit()

    def function_1_FE(self, x, le):
        return 1 - 3 * x**2 / le**2 + 2 * x**3 / le**3

    def function_1_FE_x(self, x, le):
        return -6 * x / le**2 + 6 * x**2 / le**3

    def function_1_FE_xx(self, x, le):
        return -6 / le**2 + 12 * x / le**3

    def function_1_FE_xxx(self, x, le):
        return 12 / le**3

    def function_2_FE(self, x, le):
        return x - 2 * x**2 / le + x**3 / le**2

    def function_2_FE_x(self, x, le):
        return 1 - 4 * x / le + 3 * x**2 / le**2

    def function_2_FE_xx(self, x, le):
        return -4 / le + 6 * x / le**2

    def function_2_FE_xxx(self, x, le):
        return 6 / le**2

    def function_3_FE(self, x, le):
        return 3 * x**2 / le**2 - 2 * x**3 / le**3

    def function_3_FE_x(self, x, le):
        return 6 * x / le**2 - 6 * x**2 / le**3

    def function_3_FE_xx(self, x, le):
        return 6 / le**2 - 12 * x / le**3

    def function_3_FE_xxx(self, x, le):
        return -12 / le**3

    def function_4_FE(self, x, le):
        return -(x**2) / le + x**3 / le**2

    def function_4_FE_x(self, x, le):
        return -2 * x / le + 3 * x**2 / le**2

    def function_4_FE_xx(self, x, le):
        return -2 / le + 6 * x / le**2

    def function_4_FE_xxx(self, x, le):
        return 6 / le**2

    def Construct_IW_Mat(self):
        le = self.l_x / self.ne
        self.list_lxle = np.linspace(0, le, 1000)

        self.K_el_IW = np.zeros((4, 4))  # Elementsteifigkeitsmatrix für IW

        self.K_el_IW_store = np.zeros((self.ne, 4, 4))  #

        # Start-Wert für die Index-Arrays
        start_werte = np.arange(
            0, 2 * self.ne, 2
        )  # Erstellt einen Array von Startwerten

        # Erstelle den Index-Vektor
        self.index_vector = np.array(
            [np.arange(start, start + 4) for start in start_werte]
        )

        for i in range(0, self.ne, 1):
            # Row = 0, Col = ...
            self.K_el_IW[0][0] = self.EIw * np.trapz(
                self.function_1_FE_xx(self.list_lxle, le)
                * self.function_1_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW[0][1] = self.EIw * np.trapz(
                self.function_1_FE_xx(self.list_lxle, le)
                * self.function_2_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW[0][2] = self.EIw * np.trapz(
                self.function_1_FE_xx(self.list_lxle, le)
                * self.function_3_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW[0][3] = self.EIw * np.trapz(
                self.function_1_FE_xx(self.list_lxle, le)
                * self.function_4_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            # Row = 1, Col = ...

            self.K_el_IW[1][0] = self.EIw * np.trapz(
                self.function_2_FE_xx(self.list_lxle, le)
                * self.function_1_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW[1][1] = self.EIw * np.trapz(
                self.function_2_FE_xx(self.list_lxle, le)
                * self.function_2_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW[1][2] = self.EIw * np.trapz(
                self.function_2_FE_xx(self.list_lxle, le)
                * self.function_3_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW[1][3] = self.EIw * np.trapz(
                self.function_2_FE_xx(self.list_lxle, le)
                * self.function_4_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            # Row = 2, Col = ...

            self.K_el_IW[2][0] = self.EIw * np.trapz(
                self.function_3_FE_xx(self.list_lxle, le)
                * self.function_1_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW[2][1] = self.EIw * np.trapz(
                self.function_3_FE_xx(self.list_lxle, le)
                * self.function_2_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW[2][2] = self.EIw * np.trapz(
                self.function_3_FE_xx(self.list_lxle, le)
                * self.function_3_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW[2][3] = self.EIw * np.trapz(
                self.function_3_FE_xx(self.list_lxle, le)
                * self.function_4_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            # Row = 3,  Col = ...

            self.K_el_IW[3][0] = self.EIw * np.trapz(
                self.function_4_FE_xx(self.list_lxle, le)
                * self.function_1_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW[3][1] = self.EIw * np.trapz(
                self.function_4_FE_xx(self.list_lxle, le)
                * self.function_2_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW[3][2] = self.EIw * np.trapz(
                self.function_4_FE_xx(self.list_lxle, le)
                * self.function_3_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW[3][3] = self.EIw * np.trapz(
                self.function_4_FE_xx(self.list_lxle, le)
                * self.function_4_FE_xx(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IW_store[
                i, :, :
            ] = self.K_el_IW  # Speichert die 2D Arrays in das 3D Array ab

        for n in range(0, self.ne, 1):
            for row in range(0, 4, 1):
                for col in range(0, 4, 1):
                    self.K_I[self.index_vector[n][row]][
                        self.index_vector[n][col]
                    ] += self.K_el_IW_store[n][row][col]

    def Construct_GIT_Mat(self):
        le = self.l_x / self.ne
        self.list_lxle = np.linspace(0, le, 1000)

        self.K_el_IT = np.zeros((4, 4))  # Elementsteifigkeitsmatrix für IT
        self.K_el_IT_store = np.zeros((self.ne, 4, 4))  #

        # Start-Wert für die Index-Arrays
        start_werte = np.arange(
            0, 2 * self.ne, 2
        )  # Erstellt einen Array von Startwerten

        # Erstelle den Index-Vektor
        self.index_vector = np.array(
            [np.arange(start, start + 4) for start in start_werte]
        )

        for i in range(0, self.ne, 1):
            # Row = 0, Col = ...
            self.K_el_IT[0][0] = self.GIt * np.trapz(
                self.function_1_FE_x(self.list_lxle, le)
                * self.function_1_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT[0][1] = self.GIt * np.trapz(
                self.function_1_FE_x(self.list_lxle, le)
                * self.function_2_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT[0][2] = self.GIt * np.trapz(
                self.function_1_FE_x(self.list_lxle, le)
                * self.function_3_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT[0][3] = self.GIt * np.trapz(
                self.function_1_FE_x(self.list_lxle, le)
                * self.function_4_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            # Row = 1, Col = ...

            self.K_el_IT[1][0] = self.GIt * np.trapz(
                self.function_2_FE_x(self.list_lxle, le)
                * self.function_1_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT[1][1] = self.GIt * np.trapz(
                self.function_2_FE_x(self.list_lxle, le)
                * self.function_2_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT[1][2] = self.GIt * np.trapz(
                self.function_2_FE_x(self.list_lxle, le)
                * self.function_3_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT[1][3] = self.GIt * np.trapz(
                self.function_2_FE_x(self.list_lxle, le)
                * self.function_4_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            # Row = 2, Col = ...

            self.K_el_IT[2][0] = self.GIt * np.trapz(
                self.function_3_FE_x(self.list_lxle, le)
                * self.function_1_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT[2][1] = self.GIt * np.trapz(
                self.function_3_FE_x(self.list_lxle, le)
                * self.function_2_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT[2][2] = self.GIt * np.trapz(
                self.function_3_FE_x(self.list_lxle, le)
                * self.function_3_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT[2][3] = self.GIt * np.trapz(
                self.function_3_FE_x(self.list_lxle, le)
                * self.function_4_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            # Row = 3,  Col = ...

            self.K_el_IT[3][0] = self.GIt * np.trapz(
                self.function_4_FE_x(self.list_lxle, le)
                * self.function_1_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT[3][1] = self.GIt * np.trapz(
                self.function_4_FE_x(self.list_lxle, le)
                * self.function_2_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT[3][2] = self.GIt * np.trapz(
                self.function_4_FE_x(self.list_lxle, le)
                * self.function_3_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT[3][3] = self.GIt * np.trapz(
                self.function_4_FE_x(self.list_lxle, le)
                * self.function_4_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_IT_store[
                i, :, :
            ] = self.K_el_IT  # Speichert die 2D Arrays in das 3D Array ab

        for n in range(0, self.ne, 1):
            for row in range(0, 4, 1):
                for col in range(0, 4, 1):
                    self.K_I[self.index_vector[n][row]][
                        self.index_vector[n][col]
                    ] += self.K_el_IT_store[n][row][col]

    def Construct_Mcr_Mat_1(self):
        le = self.l_x / self.ne
        self.list_lxle = np.linspace(0, le, 5)      # Muss der Nummer der Interpolationsstellen entsprechen! Aus My-Berechnung

        self.K_el_EIZ = np.zeros(
            (4, 4)
        )  # Geometrische Steifigkeitsmatrix quadratischen Lastanteil

        self.K_el_EIZ_store = np.zeros((self.ne, 4, 4))  #

        # Start-Wert für die Index-Arrays
        start_werte = np.arange(
            0, 2 * self.ne, 2
        )  # Erstellt einen Array von Startwerten

        # Erstelle den Index-Vektor
        self.index_vector = np.array(
            [np.arange(start, start + 4) for start in start_werte]
        )

        for i in range(0, self.ne, 1):
            # Row = 0, Col = ...
            
            self.K_el_EIZ[0][0] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_1_FE(self.list_lxle, le)
                * self.function_1_FE(self.list_lxle, le),
                self.list_lxle
            )

            self.K_el_EIZ[0][1] = self.EIzz* np.trapz(
                self.My_x[i]**2 * 
                self.function_1_FE(self.list_lxle, le)
                * self.function_2_FE(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_EIZ[0][2] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_1_FE(self.list_lxle, le)
                * self.function_3_FE(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_EIZ[0][3] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_1_FE(self.list_lxle, le)
                * self.function_4_FE(self.list_lxle, le),
                self.list_lxle,
            )

            # Row = 1, Col = ...

            self.K_el_EIZ[1][0] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_2_FE(self.list_lxle, le)
                * self.function_1_FE(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_EIZ[1][1] = self.EIzz* np.trapz(
                self.My_x[i]**2 * 
                self.function_2_FE(self.list_lxle, le)
                * self.function_2_FE(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_EIZ[1][2] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_2_FE(self.list_lxle, le)
                * self.function_3_FE(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_EIZ[1][3] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_2_FE(self.list_lxle, le)
                * self.function_4_FE(self.list_lxle, le),
                self.list_lxle,
            )

            # Row = 2, Col = ...

            self.K_el_EIZ[2][0] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_3_FE(self.list_lxle, le)
                * self.function_1_FE(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_EIZ[2][1] = self.EIzz* np.trapz(
                self.My_x[i]**2 * 
                self.function_3_FE(self.list_lxle, le)
                * self.function_2_FE(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_EIZ[2][2] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_3_FE(self.list_lxle, le)
                * self.function_3_FE(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_EIZ[2][3] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_3_FE(self.list_lxle, le)
                * self.function_4_FE(self.list_lxle, le),
                self.list_lxle,
            )

            # Row = 3,  Col = ...

            self.K_el_EIZ[3][0] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_4_FE(self.list_lxle, le)
                * self.function_1_FE(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_EIZ[3][1] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_4_FE(self.list_lxle, le)
                * self.function_2_FE(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_EIZ[3][2] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_4_FE(self.list_lxle, le)
                * self.function_3_FE(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_EIZ[3][3] = self.EIzz * np.trapz(
                self.My_x[i]**2 * 
                self.function_4_FE(self.list_lxle, le)
                * self.function_4_FE(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_EIZ_store[
                i, :, :
            ] = self.K_el_EIZ  # Speichert die 2D Arrays in das 3D Array ab

        for n in range(0, self.ne, 1):
            for row in range(0, 4, 1):
                for col in range(0, 4, 1):
                    self.K_II_Thet[self.index_vector[n][row]][
                        self.index_vector[n][col]
                    ] += self.K_el_EIZ_store[n][row][col]

    def Construct_Mcr_Mat_2(self):
        le = self.l_x / self.ne
        self.list_lxle = np.linspace(0, le, 5)      # Muss der Nummer der Interpolationsstellen entsprechen! Aus My-Berechnung

        self.K_el_thet_II = np.zeros(
            (4, 4)
        )  # Geometrische Steifigkeitsmatrix quadratischen Lastanteil

        self.K_el_thet_II_store = np.zeros((self.ne, 4, 4))  #

        # Start-Wert für die Index-Arrays
        start_werte = np.arange(
            0, 2 * self.ne, 2
        )  # Erstellt einen Array von Startwerten

        # Erstelle den Index-Vektor
        self.index_vector = np.array(
            [np.arange(start, start + 4) for start in start_werte]
        )

        for i in range(0, self.ne, 1):
            # Row = 0, Col = ...
            
            self.K_el_thet_II[0][0] =  np.trapz(
                self.My_x[i]*  self.r * 
                self.function_1_FE_x(self.list_lxle, le)
                * self.function_1_FE_x(self.list_lxle, le),
                self.list_lxle
            )

            self.K_el_thet_II[0][1] =  np.trapz(
                self.My_x[i]* self.r *
                self.function_1_FE_x(self.list_lxle, le)
                * self.function_2_FE_x(self.list_lxle, le),
                self.list_lxle
            )

            self.K_el_thet_II[0][2] = np.trapz(
                self.My_x[i]* self.r  *
                self.function_1_FE_x(self.list_lxle, le)
                * self.function_3_FE_x(self.list_lxle, le),
                self.list_lxle
            )

            self.K_el_thet_II[0][3] =  np.trapz(
                self.My_x[i]* self.r *
                self.function_1_FE_x(self.list_lxle, le)
                * self.function_4_FE_x(self.list_lxle, le),
                self.list_lxle
            )

            # Row = 1, Col = ...

            self.K_el_thet_II[1][0] =  np.trapz(
                self.My_x[i]* self.r *
                self.function_2_FE_x(self.list_lxle, le)
                * self.function_1_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_thet_II[1][1] =  np.trapz(
                self.My_x[i] * self.r  *
                self.function_2_FE_x(self.list_lxle, le)
                * self.function_2_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_thet_II[1][2] =  np.trapz(
                self.My_x[i]* self.r  *
                self.function_2_FE_x(self.list_lxle, le)
                * self.function_3_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_thet_II[1][3] =  np.trapz(
                self.My_x[i]* self.r  *
                self.function_2_FE_x(self.list_lxle, le)
                * self.function_4_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            # Row = 2, Col = ...

            self.K_el_thet_II[2][0] =  np.trapz(
                self.My_x[i]* self.r  *
                self.function_3_FE_x(self.list_lxle, le)
                * self.function_1_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_thet_II[2][1] =  np.trapz(
                self.My_x[i]* self.r *
                self.function_3_FE_x(self.list_lxle, le)
                * self.function_2_FE_x(self.list_lxle, le),
                self.list_lxle
            )

            self.K_el_thet_II[2][2] =  np.trapz(
                self.My_x[i]* self.r *
                self.function_3_FE_x(self.list_lxle, le)
                * self.function_3_FE_x(self.list_lxle, le),
                self.list_lxle
            )

            self.K_el_thet_II[2][3] = np.trapz(
                self.My_x[i]* self.r  *
                self.function_3_FE_x(self.list_lxle, le)
                * self.function_4_FE_x(self.list_lxle, le),
                self.list_lxle
            )

            # Row = 3,  Col = ...

            self.K_el_thet_II[3][0] =  np.trapz(
                self.My_x[i]* self.r *
                self.function_4_FE_x(self.list_lxle, le)
                * self.function_1_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_thet_II[3][1] =  np.trapz(
                self.My_x[i]* self.r  *
                self.function_4_FE_x(self.list_lxle, le)
                * self.function_2_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_thet_II[3][2] =  np.trapz(
                self.My_x[i]* self.r *
                self.function_4_FE_x(self.list_lxle, le)
                * self.function_3_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_thet_II[3][3] =  np.trapz(
                self.My_x[i]* self.r *
                self.function_4_FE_x(self.list_lxle, le)
                * self.function_4_FE_x(self.list_lxle, le),
                self.list_lxle,
            )

            self.K_el_thet_II_store[
                i, :, :
            ] = self.K_el_thet_II  # Speichert die 2D Arrays in das 3D Array ab

        for n in range(0, self.ne, 1):
            for row in range(0, 4, 1):
                for col in range(0, 4, 1):
                    self.K_II_Thet_s[self.index_vector[n][row]][
                        self.index_vector[n][col]
                    ] += self.K_el_thet_II_store[n][row][col]



    def Export_ElementStiffness_IW(self, n_elem, name):
        Matrix = self.K_el_IW_store[n_elem].round(2)
        df = pd.DataFrame(Matrix, index=[1, 2, 3, 4])
        df.to_csv(
            f"Checks/Data/ElementStiffness_{name}_{n_elem}_IW.csv",
            header=False,
            index=[1, 2, 3, 4],
        )

        Matrix = self.K_el_IT_store[n_elem].round(2)
        df = pd.DataFrame(Matrix)
        df.to_csv(
            f"Checks/Data/ElementStiffness_{name}_{n_elem}_IT.csv",
            header=False,
            index=[1, 2, 3, 4],
        )

        Matrix = self.K_el_EIZ_store[n_elem].round(2)
        df = pd.DataFrame(Matrix)
        df.to_csv(
            f"Checks/Data/ElementStiffness_{name}_{n_elem}_KIITHET.csv",
            header=False,
            index=[1, 2, 3, 4],
        )

        Matrix = self.K_el_thet_II_store[n_elem].round(2)
        df = pd.DataFrame(Matrix)
        df.to_csv(
            f"Checks/Data/ElementStiffness_{name}_{n_elem}_KIITHET_s.csv",
            header=False,
            index=[1, 2, 3, 4],
        )


    def calculate_alpha_crit(self):
        alpha_min = 0
        alpha_max = 10
        precision = 1e-3
        initial_delta_alpha = 0.1
        critical_alphas = []
        self._recursive_search(alpha_min, alpha_max, initial_delta_alpha, precision, critical_alphas)
        return critical_alphas

    def _recursive_search(self, alpha_min, alpha_max, delta_alpha, precision, critical_alphas):
        alpha_0 = alpha_min
        det0 = np.linalg.det(self.K_I - alpha_0**2 * self.K_II_Thet + alpha_0 * self.K_II_Thet_s) # 
        for i in range(1, int((alpha_max - alpha_min) / delta_alpha) + 1):
            alpha_0 = alpha_min + i * delta_alpha
            det1 = np.linalg.det(self.K_I - alpha_0**2 * self.K_II_Thet + alpha_0 * self.K_II_Thet_s)# 
            try:
                val = det0 * det1 
            except:
                val = det0 * det1 / 1e50
            if val < 0:  # Vorzeichenwechsel erkannt
                if delta_alpha <= precision:
                    # Eigenwert gefunden, Mittelpunkt des Intervalls speichern
                    alpha_crit = (alpha_0 - delta_alpha / 2)
                    critical_alphas.append(alpha_crit)
                    print(f"Eigenwert gefunden: alpha_crit = {alpha_crit}, Mcrit = {alpha_crit * self.My_x.max()}")
                else:
                    # Weitere Suche in diesem Intervall mit feinerer Schrittweite
                    self._recursive_search(alpha_0 - delta_alpha, alpha_0, delta_alpha / 10, precision, critical_alphas)
            det0 = det1

BDK = Biegedrillknicken()
BDK.Calculate_All()
BDK.Export_ElementStiffness_IW(1, "Unittest")


print(tabulate(BDK.K_el_IW))
print(tabulate(BDK.K_el_IT))
print(BDK.r)



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy.interpolate import interp1d
from Materialkennwerte_Beton import Baustoffe
from Querschnittsbreite import PolygonSection

class M_Kappa_Cross_Section:
    def __init__(
        self,
        _LoadingType="csv",
        Iter="Bruch",
        Reinforcement="csv",
        Querschnittsbemessung="Polygon",
    ):
        # Init design values for loading
        self.LoadingType = _LoadingType
        # Geometric parameters for the design
        self.Bewehrung = Reinforcement

        self.readBaustoffe()
        self.Querschnittswerte()
        self.Bewehrungsangaben()
        self.ReadLoading()

        print("height", self.h_c)
        print("width", self.b_c)

        # Latex-Ausgabe

        self.results_A = []
        self.results_B = []
        self.results_B_Iterate = []
        self.results_C = []
        self.results_C_Iterate = []
        self.results_D = []
        self.results_D_Iterate = []


        self.kappa_list = []
        self.M_list     = []
        self.state_list = []   # optional: A, B, C, D


        # Bemessungswerte der Baustoffwerte

        self.Point_A()
        self.Point_B()
        self.Point_B_Iterate()
        self.Point_C()
        self.Point_C_Iterative()
        # self.Point_D()

    def readBaustoffe(self):
        csv_read_beton = pd.read_csv("Materialparameter/Beton.csv")
        csv_read_stahl = pd.read_csv("Materialparameter/Betonstahl.csv")

        print(csv_read_beton["traeger"])
        self.Baustoff = Baustoffe(
            [csv_read_beton["fck"].iloc[0], csv_read_stahl["fyk[MPa]"].iloc[0]],
            csv_read_beton["h_c"].iloc[0],
            csv_read_beton["b_c"].iloc[0],
            csv_read_beton["traeger"].iloc[0],
            csv_read_beton["t0"].iloc[0],
            csv_read_beton["t"].iloc[0],
        )

    def Querschnittswerte(self, _height_test=0.5):
        # Example usage
        df = pd.read_csv("Polygon/Polygon.csv")
        self.vertices = df.values.tolist()

        self.polygon = PolygonSection(self.vertices)

        # Calculate the height of the polygon based on y-values

        y_values = [vertex[1] for vertex in self.vertices]
        self.height = abs(max(y_values) - min(y_values))
        self.z_cu = self.height - self.polygon.centroid[1]
        self.z_co = self.height - self.z_cu

        # Define the rotation angle
        self.angle_degrees = 0
        self.polygon.rotate(self.angle_degrees)

        height = self.height


        self.num_stripes = 1000
        self.section_width_list = [
            self.polygon.calculate_section_width_at_height(
                height / (self.num_stripes) * (i + 0.5)
            )
            for i in range(0, self.num_stripes, 1)
        ]
        self.section_height_list = [
            height / (self.num_stripes) * (i + 0.5)
            for i in range(0, self.num_stripes, 1)
        ]
        self.delta_height = height / self.num_stripes

        self.h_c = self.height
        self.b_c = self.polygon.calculate_section_width_at_height(_height_test)

        self.A = self.polygon.A
    
    def PlotCrossSection(self, _height_test=0.25):
        # Define the height at which to calculate the section width
        height = _height_test
        section_width = self.polygon.calculate_section_width_at_height(height)
        print(
            f"Section width at height {height} after rotating by {self.angle_degrees} degrees: {section_width}"
        )

        # Plot the polygon and the horizontal line
        self.polygon.plot(height)

    def Bewehrungsangaben(self):
        if self.Bewehrung == "csv":
            df = pd.read_csv("Bewehrung/Linienbewehrung.csv")
            print(df)
            for i in range(0, len(df["Lage"]), 1):
                Lage = df["Lage"][i]
                if Lage == "Unten":
                    self.d_s1 = (
                        df["dsl [m]"][i] * 0.5 + df["cnom [m]"][i] + df["dsw [m]"][i]
                    )
                    self.A_s1 = df["As [cm**2]"][i] * 0.01**2
                elif Lage == "Oben":
                    self.d_s2 = (
                        df["dsl [m]"][i] * 0.5 + df["cnom [m]"][i] + df["dsw [m]"][i]
                    )
                    self.A_s2 = df["As [cm**2]"][i] * 0.01**2

            self.z_ds1 = self.z_cu - self.d_s1
            self.z_ds2 = self.z_co - self.d_s2

    def ReadLoading(self):
        if self.LoadingType == "csv":
            df = pd.read_csv("Lasteingabe/Lasten.csv")
            print(df)
            self.Zugseite = None
            for i in range(0, len(df["Grenzzustand"]), 1):
                if df["Grenzzustand"][i] == "GZT":
                    self.NEd_GZT = df["NEd in [MN]"][i]
                    self.MEd_GZT = df["MEd in [MNm]"][i]
                elif df["Grenzzustand"][i] == "GZG":
                    self.NEd_GZG = df["NEd in [MN]"][i]
                    self.MEd_GZG = df["MEd in [MNm]"][i]

            if self.MEd_GZT >= 0:
                self.MEds_GZT = abs(
                    self.MEd_GZT - self.NEd_GZT * self.z_ds1
                )  # Bezug auf die Biegezugbewehrung (Hier UNTEN)
                self.Zugseite_GZT = "UNTEN"
                self.d = self.height - self.d_s1
            elif self.MEd_GZT < 0:
                self.MEds_GZT = abs(
                    self.MEd_GZT + self.NEd_GZT * self.z_ds2
                )  # Bezug auf die Biegezugbewehrung (Hier OBEN)
                self.Zugseite_GZT = "OBEN"
                self.d = self.height - self.d_s2

            if self.MEd_GZG >= 0:
                self.MEds_GZG = self.MEd_GZG - self.NEd_GZG * self.z_ds1
                self.Zugseite_GZG = "UNTEN"
                self.d = self.height - self.d_s1
            elif self.MEd_GZG < 0:
                self.MEds_GZG = self.MEd_GZG + self.NEd_GZG * self.z_ds2
                self.Zugseite_GZG = "OBEN"
                self.d = self.height - self.d_s2

        elif self.LoadingType == "csv_druckglied":
            df = pd.read_csv("Lasteingabe/Lasten_Druckglied.csv")
            print(df)
            self.Zugseite = None
            for i in range(0, len(df["Grenzzustand"]), 1):
                if df["Grenzzustand"][i] == "GZT":
                    self.NEd_GZT = df["NEd in [MN]"][i]
                    self.MEd1_GZT = df["MEd1 in [MNm]"][i]
                    self.MEd2_GZT = df["MEd2 in [MNm]"][i]
                elif df["Grenzzustand"][i] == "GZG":
                    self.NEd_GZG = df["NEd in [MN]"][i]
                    self.MEd1_GZG = df["MEd1 in [MNm]"][i]
                    self.MEd2_GZG = df["MEd2 in [MNm]"][i]

            if self.MEd1_GZT >= 0:
                self.Zugseite_GZT = "UNTEN"
                self.d = self.height - self.d_s1
            elif self.MEd1_GZT < 0:
                self.Zugseite_GZT = "OBEN"
                self.d = self.height - self.d_s2

            if self.MEd1_GZG >= 0:
                self.Zugseite_GZG = "UNTEN"
                self.d = self.height - self.d_s1
            elif self.MEd1_GZG < 0:
                self.Zugseite_GZG = "OBEN"
                self.d = self.height - self.d_s2

        else:
            self.NEd_GZT = float(
                input("Geben Sie die Normalkraft NEd im GZT in [MN] ein: \n")
            )
            self.MEd_GZT = float(
                input("Geben Sie das Biegemoment im GZT in [MN] ein: \n")
            )
            if self.MEd_GZT >= 0:
                self.MEds_GZT = self.MEd_GZT - self.NEd_GZT * self.z_ds1
            elif self.MEd_GZT < 0:
                self.MEds_GZT = self.MEd_GZT + self.NEd_GZT * self.z_ds1

            if self.MEd_GZG >= 0:
                self.MEds_GZG = self.MEd_GZG - self.NEd_GZG * self.z_ds1
            elif self.MEd_GZG < 0:
                self.MEds_GZG = self.MEd_GZG + self.NEd_GZG * self.z_ds1

        # Export loading parameters to the output folder
        self.zsi_GZT = None
        self.zsi_GZG = None

        if self.Zugseite_GZT == "UNTEN":
            self.zsi_GZT = self.z_ds1
        else:
            self.zsi_GZT = self.z_ds2

        if self.Zugseite_GZG == "UNTEN":
            self.zsi_GZG = self.z_ds1
        else:
            self.zsi_GZG = self.z_ds2

    def Nonlinear_Material_Law(self, eps_c):
        # Berechnung von eta und k
        eta = abs(eps_c / (self.Baustoff.eps_c1 * 1e-3))
        k = (
            1.05
            * self.Baustoff.Ecm
            / 1.5
            * abs(self.Baustoff.eps_c1 * 1e-3)
            / self.Baustoff.fcd
        )

        # Berechnung von sigma_c

        sigma_c = self.Baustoff.fcd * (k * eta - eta**2) / (1 + (k - 2) * eta)

        return eta, k, sigma_c

    def Point_A(self):
        """
        Calculates the strain for the case, that the cross section is cracking at one edge
        """

        self.Wco = self.polygon.I_yy / self.z_co
        self.Wcu = self.polygon.I_yy / self.z_cu
        self.A = self.polygon.A

        self.M_cr_o = self.Wco * (self.Baustoff.fctm - self.NEd_GZT / self.A)
        self.M_cr_u = self.Wcu * (self.Baustoff.fctm - self.NEd_GZT / self.A)

        print("Area", self.polygon.A)
        print("Trägheitsmoment ", self.polygon.I_yy)
        print("Widerstandsmoment ", self.Wco)
        print("Zugfestigkeit ", self.Baustoff.fctm)

        self.M_A = min(self.M_cr_o, self.M_cr_u)
        self.kappa_A = self.M_A / (self.Baustoff.Ecm * self.polygon.I_yy)

        # Save results
        self.results_A.append(("I_y", self.polygon.I_yy))
        self.results_A.append(("Wco", self.Wco))
        self.results_A.append(("Wcu", self.Wcu))
        self.results_A.append(("A", self.A))
        self.results_A.append(("M_cr_o", self.M_cr_o))
        self.results_A.append(("M_cr_u", self.M_cr_u))
        self.results_A.append(("M_A", self.M_A))
        self.results_A.append(("Kappa_A", self.kappa_A))

        self.latex_table_A = self.results_to_latex_table(
            self.results_A, ["Parameter", "Wert"], "Results of Point A"
        )

        self.kappa_A = self.kappa_A
        self.M_A     = self.M_A

        self.kappa_list.append(self.kappa_A)
        self.M_list.append(self.M_A)
        self.state_list.append("A")



    def Point_B(self):
        """
        Point B is the case, where the compression reinforcement reaches its \n
        yielding stress. \n
        The iterations starts at the yielding strains of -2.174e-3 in both compression and \n
        tension reinforcement.
        """
        N_Rd_B = self.NEd_GZT - 10
        iter = 0

        epsilon_s2 = -2.174e-3
        epsilon_s1 = -2.174e-3
        epsilon_c2 = epsilon_s2 + (
            (epsilon_s2 - epsilon_s1) / (self.h_c - self.d_s1 - self.d_s2) * (self.d_s2)
        )

        print(epsilon_c2)

        N_Rd_B_list = []

        x = self.h_c

        while (N_Rd_B < self.NEd_GZT) and (iter < 1000) and (epsilon_c2 >= -0.0035):
            eta = self.Nonlinear_Material_Law(epsilon_c2)[0]
            k = self.Nonlinear_Material_Law(epsilon_c2)[1]
            sigma_c = self.Nonlinear_Material_Law(epsilon_c2)[2]

            epsilon_c_list = np.linspace(0, epsilon_c2, 100)

            sigma_c_list = self.Nonlinear_Material_Law(epsilon_c_list)[2]

            x_list = np.linspace(0, x, 100)

            if abs(N_Rd_B - self.NEd_GZT) > 1e-1:
                epsilon_s1 += 1e-4
            else:
                epsilon_s1 += 1e-5

            epsilon_c2 = epsilon_s2 + (
                (epsilon_s2 - epsilon_s1)
                / (self.h_c - self.d_s1 - self.d_s2)
                * (self.d_s2)
            )

            N_cRd_B = np.trapz(sigma_c_list, x_list) * self.b_c * (-1)

            if abs(epsilon_s2) <= 2.174e-3:
                N_s2_Rd_B = epsilon_s2 * self.A_s2 * self.Baustoff.Es
            else:
                N_s2_Rd_B = -self.A_s2 * self.Baustoff.fyd
            if abs(epsilon_s1) <= 2.174e-3:
                N_s1_Rd_B = epsilon_s1 * self.A_s1 * self.Baustoff.Es
            else:
                N_s1_Rd_B = self.Baustoff.fyd * self.A_s1

            # Berechnung des Abstands der Druckresultierenden von DNL

            N_Rd_B_list.append(N_cRd_B)

            dx = x_list[1] - x_list[0]

            # Berechnung der Fläche unter der Kurve (Flächenmoment 0. Grades)
            A = np.sum(sigma_c_list * dx)

            # Berechnung des 1. Flächenmoments um den Ursprung
            S = np.sum(sigma_c_list * x_list * dx)

            # Berechnung des Schwerpunkts
            a = S / A

            N_Rd_B = N_cRd_B + N_s2_Rd_B + N_s1_Rd_B

            iter += 1

        print("eps s1", epsilon_s1)
        print("eps s2", epsilon_s2)
        print("eps c2", epsilon_c2)

        print("NRd_B", N_Rd_B)
        print("NRd_s1", N_s1_Rd_B)
        print("NRd_s2", N_s2_Rd_B)

        self.M_B = (
            abs(N_cRd_B) * (self.z_cu - a)
            + abs(N_s2_Rd_B) * (self.z_cu - self.d_s2)
            + abs(N_s1_Rd_B) * (self.z_co - self.d_s1)
        )

        self.kappa_B = (epsilon_s1 - epsilon_c2) / self.d

        if epsilon_c2 < -0.0035:
            self.M_B = None
            self.kappa_B = None

        self.results_B.append(("M_B", self.M_B))
        self.results_B.append(("Kappa_B", self.kappa_B))


        if self.M_B is not None:
            self.kappa_list.append(self.kappa_B)
            self.M_list.append(self.M_B)
            self.state_list.append("B")

    def Point_B_Iterate(self):
        """
        Point B is the case, where the compression reinforcement reaches its \n
        yielding stress. \n
        The iterations starts at the yielding strains of -2.174e-3 in both compression and \n
        tension reinforcement. \n
        This function is the iterative representative for general cross sections.
        """
        N_Rd_B = self.NEd_GZT - 10
        iter = 0

        epsilon_s2 = -2.174e-3
        epsilon_s1 = -2.174e-3
        epsilon_c2 = epsilon_s2 + (
            (epsilon_s2 - epsilon_s1) / (self.h_c - self.d_s1 - self.d_s2) * (self.d_s2)
        )

        N_Rd_B_list = []
        x = self.h_c

        while (N_Rd_B < self.NEd_GZT) and (iter < 1000) and (epsilon_c2 >= -0.0035):

            eps_0 = epsilon_s2 - (epsilon_s2 - epsilon_s1) / (
                self.h_c - self.d_s1 - self.d_s2
            ) * (self.h_c - self.d_s2)

            delta_eps_h = epsilon_c2 - eps_0

            print(delta_eps_h)

            N_cRd_B = 0
            F_cd_list = np.zeros(1000)

            for i in range(0, len(self.section_height_list), 1):
                epsilon_i = eps_0 + delta_eps_h / (self.num_stripes) * (i)
                if epsilon_i < 0:
                    sigma_c = self.Nonlinear_Material_Law(epsilon_i)[2]
                else:
                    sigma_c = 0

                F_cd_i = sigma_c * self.delta_height * self.section_width_list[i]
                F_cd_list[i] = F_cd_i

            N_cRd_B = sum(F_cd_list) * (-1)

            if abs(N_Rd_B - self.NEd_GZT) > 1e-1:
                epsilon_s1 += 1e-4
            else:
                epsilon_s1 += 1e-5

            epsilon_c2 = epsilon_s2 + (
                (epsilon_s2 - epsilon_s1)
                / (self.h_c - self.d_s1 - self.d_s2)
                * (self.d_s2)
            )

            if abs(epsilon_s2) <= 2.174e-3:
                N_s2_Rd_B = epsilon_s2 * self.A_s2 * self.Baustoff.Es
            else:
                N_s2_Rd_B = -self.A_s2 * self.Baustoff.fyd
            if abs(epsilon_s1) <= 2.174e-3:
                N_s1_Rd_B = epsilon_s1 * self.A_s1 * self.Baustoff.Es
            else:
                N_s1_Rd_B = self.Baustoff.fyd * self.A_s1

            N_Rd_B = N_cRd_B + N_s2_Rd_B + N_s1_Rd_B

            # print("N_c_RdB", N_cRd_B)
            # print("N_s1", N_s1_Rd_B)
            # print("N_s2", N_s2_Rd_B)
            # print("Ned",self.NEd_GZT)

            iter += 1
        print("H-GGW", abs(N_Rd_B - self.NEd_GZT))
        print("eps s1 - ITER", epsilon_s1)
        print("eps s2 - ITER", epsilon_s2)
        print("eps c2 - ITER", epsilon_c2)

        print("NRd_B - ITER", N_Rd_B)
        print("NRd_s1 - ITER", N_s1_Rd_B)
        print("NRd_s2 - ITER", N_s2_Rd_B)

        # self.M_B_Iterate = (
        #     abs(N_cRd_B) * (self.z_cu - a)
        #     + abs(N_s2_Rd_B) * (self.z_cu - self.d_s2)
        #     + abs(N_s1_Rd_B) * (self.z_co - self.d_s1)
        # )

        # self.kappa_B_Iterate = (epsilon_s1 - epsilon_c2) / self.d

        # if epsilon_c2 < -0.0035:
        #     self.M_B = None
        #     self.kappa_B = None

        # self.results_B_Iterate.append(("M_B", self.M_B_Iterate))
        # self.results_B_Iterate.append(("Kappa_B", self.kappa_B_Iterate))

        # plt.plot(F_cd_list)
        # plt.show()

    def Point_C(self):
        """
        Point C represents the point in the M-Kappa-Law, where \n
        the tensional reinforcement reaches the yielding stresses. \n
        Therefore the iteration starts at 2.174e-3 \n
        The iteration is performed over the strain of the compressional reinforcement \n

        """
        print("Point C")
        N_Rd_C = self.NEd_GZT + 1
        iter = 0

        epsilon_s1 = +2.174e-3
        epsilon_s2 = 0

        epsilon_c2 = epsilon_s1 + (
            (epsilon_s2 - epsilon_s1)
            / (self.h_c - self.d_s1 - self.d_s2)
            * (self.h_c - self.d_s1)
        )

        x = epsilon_c2 / (epsilon_c2 - epsilon_s1) * self.d

        print()

        while (N_Rd_C > self.NEd_GZT) and (iter < 1000) and (abs(epsilon_c2) <= 3.5e-3):
            
            eta = self.Nonlinear_Material_Law(epsilon_c2)[0]
            k = self.Nonlinear_Material_Law(epsilon_c2)[1]
            sigma_c = self.Nonlinear_Material_Law(epsilon_c2)[2]

            epsilon_c_list = np.linspace(0, epsilon_c2, 100)

            sigma_c_list = self.Nonlinear_Material_Law(epsilon_c_list)[2]

            x_list = np.linspace(0, x, 100)

            if abs(N_Rd_C - self.NEd_GZT) > 1e-1:
                epsilon_s2 -= 1e-4
            else:
                epsilon_s2 -= 1e-5

            epsilon_c2 = epsilon_s1 + (
                (epsilon_s2 - epsilon_s1)
                / (self.h_c - self.d_s1 - self.d_s2)
                * (self.h_c - self.d_s1)
            )

            x = epsilon_c2 / (epsilon_c2 - epsilon_s1) * self.d

            N_cRd_C = np.trapz(sigma_c_list, x_list) * self.b_c * (-1)

            if abs(epsilon_s2) <= 2.174e-3:
                N_s2_Rd_C = epsilon_s2 * self.A_s2 * self.Baustoff.Es
            else:
                N_s2_Rd_C = -self.A_s2 * self.Baustoff.fyd
            if abs(epsilon_s1) <= 2.174e-3:
                N_s1_Rd_C = epsilon_s1 * self.A_s1 * self.Baustoff.Es
            else:
                N_s1_Rd_C = self.Baustoff.fyd * self.A_s1

            N_Rd_C = N_cRd_C + N_s2_Rd_C + N_s1_Rd_C

            # Berechnung des Abstands der Druckresultierenden von DNL

            dx = x_list[1] - x_list[0]

            # Berechnung der Fläche unter der Kurve (Flächenmoment 0. Grades)
            A = np.sum(sigma_c_list * dx)

            # Berechnung des 1. Flächenmoments um den Ursprung
            S = np.sum(sigma_c_list * x_list * dx)

            # Berechnung des Schwerpunkts
            a = S / A

            iter += 1

        print("H-GGW", abs(N_Rd_C - self.NEd_GZT))
        print("eps s1 ", epsilon_s1)
        print("eps s2 ", epsilon_s2)
        print("eps c2 ", epsilon_c2)

        print("NRd_C ", N_Rd_C)
        print("NRd_s1 ", N_s1_Rd_C)
        print("NRd_s2 ", N_s2_Rd_C)

        plt.plot([epsilon_s1,epsilon_s2,epsilon_c2],[0,self.d,self.height])
        plt.show()

        self.M_C = (
            abs(N_cRd_C) * (self.z_cu - x + a)
            + abs(N_s2_Rd_C) * (self.z_cu - self.d_s2)
            + abs(N_s1_Rd_C) * (self.z_co - self.d_s1)
        )

        self.kappa_C = (epsilon_s1 - epsilon_c2) / self.d

        self.results_C.append(("M_C", self.M_C))
        self.results_C.append(("Kappa_C", self.kappa_C))
    
        self.kappa_list.append(self.kappa_C)
        self.M_list.append(self.M_C)
        self.state_list.append("C")


    def Point_C_Iterative(self):
        N_Rd_C = self.NEd_GZT + 1
        iter = 0

        epsilon_s1 = +2.174e-3
        epsilon_s2 = 0

        epsilon_c2 = epsilon_s1 + (
            (epsilon_s2 - epsilon_s1)
            / (self.h_c - self.d_s1 - self.d_s2)
            * (self.h_c - self.d_s1)
        )

        x = epsilon_c2 / (epsilon_c2 - epsilon_s1) * self.d


        while (N_Rd_C > self.NEd_GZT) and (iter < 1000) and (abs(epsilon_c2) <= 3.5e-3):
            

            eps_0 = epsilon_s2 + (abs(epsilon_s2 - epsilon_s1)) / (
                self.h_c - self.d_s1 - self.d_s2
            ) * (self.h_c - self.d_s2)

            delta_eps_h = epsilon_c2 - eps_0

            x = epsilon_c2 / (epsilon_c2 - epsilon_s1) * self.d

            N_cRd_C = 0
            F_cd_list = np.zeros(1000)

            eps_list = []

            for i in range(0, len(self.section_height_list), 1):
                
                epsilon_i = eps_0 + delta_eps_h / (self.num_stripes) * (i+1.5)

                if epsilon_i <= 0:
                    sigma_c = self.Nonlinear_Material_Law(epsilon_i)[2]
                else:
                    sigma_c = 0

                F_cd_i = sigma_c * self.delta_height * self.section_width_list[i]

                F_cd_list[i] = F_cd_i

            N_cRd_C = F_cd_list.sum() * (-1)

            if abs(epsilon_s2) <= 2.174e-3:
                N_s2_Rd_C = epsilon_s2 * self.A_s2 * self.Baustoff.Es
            else:
                N_s2_Rd_C = -self.A_s2 * self.Baustoff.fyd

            if abs(epsilon_s1) <= 2.174e-3:
                N_s1_Rd_C = epsilon_s1 * self.A_s1 * self.Baustoff.Es
            else:
                N_s1_Rd_C = self.Baustoff.fyd * self.A_s1

            N_Rd_C = N_cRd_C + N_s2_Rd_C + N_s1_Rd_C

            if abs(N_Rd_C - self.NEd_GZT) > 1e-1:
                epsilon_s2 -= 1e-4
            else:
                epsilon_s2 -= 1e-5

            epsilon_c2 = epsilon_s1 + (
                (epsilon_s2 - epsilon_s1)
                / (self.h_c - self.d_s1 - self.d_s2)
                * (self.h_c - self.d_s1)
            )

            iter += 1




        print("H-GGW", abs(N_Rd_C - self.NEd_GZT))
        print("eps s1 - ITER", epsilon_s1)
        print("eps s2 - ITER", epsilon_s2)
        print("eps c2 - ITER", epsilon_c2)

        print("NRd_C - ITER", N_Rd_C)
        print("NRd_s1 - ITER", N_s1_Rd_C)
        print("NRd_s2 - ITER", N_s2_Rd_C)
        

        # self.M_C_Iter = (
        #     abs(N_cRd_C) * (self.z_cu - x + a)
        #     + abs(N_s2_Rd_C) * (self.z_cu - self.d_s2)
        #     + abs(N_s1_Rd_C) * (self.z_co - self.d_s1)
        # )

        # self.kappa_C_Iter = (epsilon_s1 - epsilon_c2) / self.d

        # self.results_C_Iterate.append(("M_C", self.M_C_Iter))
        # self.results_C_Iterate.append(("Kappa_C", self.kappa_Iter))

    def Point_D(self):
        """
        Point D represents the point in the M-Kappa-Law, where \n
        the concrete reaches its breaking strain. \n
        Therefore the iteration starts at -3.50e-3 for the concrete strain epsilon_c2 \n
        The iteration is performed over the strain of the tensional reinforcement \n

        """
        N_Rd_D = self.NEd_GZT - 1
        iter = 0

        epsilon_s1 = 0

        epsilon_c2 = -3.5e-3
        epsilon_s2 = epsilon_c2 + (epsilon_c2 - epsilon_s1) / (self.d) * (self.d_s2)

        x = epsilon_c2 / (epsilon_c2 - epsilon_s1) * self.d

        while (N_Rd_D < self.NEd_GZT) and (iter < 10000):
            eta = self.Nonlinear_Material_Law(epsilon_c2)[0]
            k = self.Nonlinear_Material_Law(epsilon_c2)[1]
            sigma_c = self.Nonlinear_Material_Law(epsilon_c2)[2]

            epsilon_c_list = np.linspace(0, epsilon_c2, 100)

            sigma_c_list = self.Nonlinear_Material_Law(epsilon_c_list)[2]

            x_list = np.linspace(0, x, 100)

            if abs(N_Rd_D - self.NEd_GZT) > 1e-1:
                epsilon_s1 += 1e-4
            else:
                epsilon_s1 += 1e-5

            print("epsilon s2", epsilon_s2)

            epsilon_s2 = epsilon_c2 + (epsilon_c2 - epsilon_s1) / (self.d) * (self.d_s2)

            x = epsilon_c2 / (epsilon_c2 - epsilon_s1) * self.d

            N_cRd_D = np.trapz(sigma_c_list, x_list) * self.b_c * (-1)
            if abs(epsilon_s2) <= 2.174e-3:
                N_s2_Rd_D = epsilon_s2 * self.A_s2 * self.Baustoff.Es
            else:
                N_s2_Rd_D = -self.A_s2 * self.Baustoff.fyd
            if abs(epsilon_s1) <= 2.174e-3:
                N_s1_Rd_D = epsilon_s1 * self.A_s1 * self.Baustoff.Es
            else:
                N_s1_Rd_D = self.Baustoff.fyd * self.A_s1

            N_Rd_D = N_cRd_D + N_s2_Rd_D + N_s1_Rd_D

            print("N_D", N_cRd_D)
            print("NDs2", N_s2_Rd_D)
            print("NDs1", N_s1_Rd_D)

            print("NDRd", N_Rd_D)

            print("NEd", self.NEd_GZT)

            print(N_Rd_D > self.NEd_GZT)

            print(epsilon_c2)

            # Berechnung des Abstands der Druckresultierenden von DNL

            dx = x_list[1] - x_list[0]

            # Berechnung der Fläche unter der Kurve (Flächenmoment 0. Grades)
            A = np.sum(sigma_c_list * dx)

            # Berechnung des 1. Flächenmoments um den Ursprung
            S = np.sum(sigma_c_list * x_list * dx)

            # Berechnung des Schwerpunkts
            a = S / A

            iter += 1

        print("a", a)
        print("x", x)

        self.M_D = (
            abs(N_cRd_D) * (self.z_cu - x + a)
            + abs(N_s2_Rd_D) * (self.z_cu - self.d_s2)
            + abs(N_s1_Rd_D) * (self.z_co - self.d_s1)
        )

        print(self.M_D)

        self.kappa_D = (epsilon_s1 - epsilon_c2) / self.d

        self.results_D.append(("M_D", self.M_D))
        self.results_D.append(("Kappa_D", self.kappa_D))

        self.kappa_list.append(self.kappa_D)
        self.M_list.append(self.M_D)
        self.state_list.append("D")


    def Point_D_Iterative(self):
        print("Point D Iterative")

    def Interpolation_Function(self, _M_Value):
        if self.kappa_B == None or round(self.A_s2, 0) == 0:
            try:
                kappa = [0, self.kappa_A, self.kappa_C, self.kappa_D]
                M = [0, self.M_A, self.M_C, self.M_D]
            except:
                kappa = [0, self.kappa_A, self.kappa_C]
                M = [0, self.M_A, self.M_C]
        else:
            try:
                kappa = [0, self.kappa_A, self.kappa_B, self.kappa_C]
                M = [0, self.M_A, self.M_B, self.M_C]
            except:
                kappa = [0, self.kappa_A, self.kappa_B, self.kappa_C]
                M = [0, self.M_A, self.M_B, self.M_C]

        moment_to_kappa_function = interp1d(M, kappa, kind="linear")

        # Gegebenen Momentenwert, für den du den kappa-Wert finden möchtest
        M_value = _M_Value  # Beispielwert für das Moment

        # Berechne den zugehörigen kappa-Wert
        kappa_value = moment_to_kappa_function(M_value)

        # Einzelwert ausgeben
        # print(f"Der zugehörige kappa-Wert für M = {M_value} ist kappa = {kappa_value}")

        # Plot der Original-Stützpunkte und der interpolierten Werte
        kappa_new = np.linspace(min(kappa), max(kappa), 100)
        M_interpolated = interp1d(kappa, M, kind="linear")(
            kappa_new
        )  # Interpolierte Momentenwerte


        return kappa_value, M_value, kappa,kappa_new,M_interpolated,M
    
    def Plot_Interpolation_M_Kappa(self, kappa_value, M_value, kappa,kappa_new,M_interpolated,M):
        plt.plot(kappa, M, "o", label="Stützpunkte")  # Original-Stützpunkte
        plt.plot(
            kappa_new, M_interpolated, "-", label="Interpolierte Werte"
        )  # Interpolierte Werte

        # Einzelnen interpolierten Punkt für den Momentenwert markieren
        plt.plot(kappa_value, M_value, "ro", label=f"kappa-Wert bei M={M_value:.2f}")
        plt.text(
            kappa_value,
            M_value,
            f"kappa = {kappa_value:.2e}, M = {M_value:.2f} ",
            color="red",
            verticalalignment="bottom",
        )

        # Legende und Plot anzeigen

        plt.legend()
        plt.show(block=False)  # block=False sorgt dafür, dass das Skript nicht anhält

        # 3 Sekunden warten
        plt.pause(5)

        # Fenster schließen
        plt.close()

    def results_to_latex_table(self, results, headers, caption):
        # Initialisiere LaTeX-Code mit Table- und Tabular-Umgebung
        latex_code = (
            "\\begin{table}[htbp]\n\\centering\n\\begin{tabular}{|"
            + "|".join(["c"] * len(headers))
            + "|}\n\\hline\n"
        )

        # Füge die Kopfzeile hinzu
        latex_code += " & ".join(headers) + " \\\\ \\hline\n"

        # Füge die Datenzeilen hinzu
        for result in results:
            # Überprüfen, ob alle Werte in der Zeile numerisch sind, bevor sie formatiert werden
            formatted_result = []
            for val in result:
                if isinstance(val, float):
                    formatted_result.append(f"{val:.4f}")
                else:
                    formatted_result.append(str(val))

            latex_code += " & ".join(formatted_result) + " \\\\ \\hline\n"

        # Schließe die Tabular- und Table-Umgebung
        latex_code += "\\end{tabular}\n\\caption{" + caption + "}\n\\end{table}\n"

        return latex_code

    def generate_latex_plot(
        self,
        data_x,
        data_y,
        xlabel="x",
        ylabel="y",
        plot_title="Plot Title",
        plot_style="thick,blue",
        caption="Plot caption",
        connect_points=True,
    ):
        # Start des LaTeX-Codes
        latex_code = "\\begin{figure}[htbp]\n\\centering\n"

        # pgfplots Umgebung
        latex_code += "\\begin{tikzpicture}\n\\begin{axis}[\n"
        latex_code += f"title={{{plot_title}}},\n"
        latex_code += f"xlabel={{{xlabel}}},\n"
        latex_code += f"ylabel={{{ylabel}}},\n"
        latex_code += "grid=major,\n]\n"

        # Plotdaten
        plot_option = plot_style
        if connect_points:
            plot_option += ",sharp plot"

        latex_code += f"\\addplot[{plot_option}] coordinates {{\n"
        for x, y in zip(data_x, data_y):
            latex_code += f"({x},{y})\n"
        latex_code += "};\n"

        # Ende der pgfplots Umgebung
        latex_code += "\\end{axis}\n\\end{tikzpicture}\n"

        # Caption und Ende der Figure-Umgebung
        latex_code += f"\\caption{{{caption}}}\n"
        latex_code += "\\end{figure}\n"

        return latex_code

    def get_M_kappa_curve(self):
        kappa = np.array(self.kappa_list)
        M     = np.array(self.M_list)

        idx = np.argsort(kappa)
        return kappa[idx], M[idx]

    def plot_M_kappa(self):
        kappa, M = self.get_M_kappa_curve()

        plt.figure()
        plt.plot(kappa, M, "-o", lw=2)
        plt.xlabel(r"$\kappa$ [1/m]")
        plt.ylabel(r"$M$ [MNm]")
        plt.grid(True)
        plt.title("Moment–Krümmungs-Beziehung")
        plt.show()



Baumgart = M_Kappa_Cross_Section()

print(Baumgart.section_width_list)

Baumgart.PlotCrossSection()

Baumgart.Interpolation_Function(0.20)

Baumgart.plot_M_kappa()


# # Speicher den LaTeX-Code in einer .tex-Datei
# with open("Output/Baumgart.tex", "w") as tex_file:
#     tex_file.write(Image )


